!******************************************************************************
!******************************** PROGRAM COMP_PIP ****************************
!******************************************************************************
! v. 040726
!******************************************************************************
!COMPILER: f90 comp_PIP.f90 -lnag -lmathlib -lkernlib -o comp_PIP.x
!
!This program calculates the composition of various plasmas within the 
!partially ionized plasma (PIP) model. For details see
! R. Redmer, PRE 59 (1999) 1073; S. Kuhlbrodt, PRE 62 (2000) 7191; 
! S. Kuhlbrodt, Doktoral Thesis, Rostock 2003 
!
!Input files are "elementfile" and "radienfile" containing information about 
!ionization energies and polarization, as well as about ionic radii of the 
!elements for which the composition can be calculated. "gridfile" gives 
!information about densities and temperatures for which the composition shall 
!be calculated. 
!The output file "outcompfile" can serve as input for calculation of transport 
!coefficients, see trans_LRT.f90. It contains information about fractions of 
!ions above a ratio n_Z/n_heavy >= 10^{-6}.
!
!
! S. Kuhlbrodt 2003
!******************************************************************************

!used modules
include "mod_constants.f90"
include "mod_basics.f90"
include "mod_fermis.f90"
include "mod_math.f90"
include "mod_comp.f90"

!******************************************************************************
!******************************************************************************

program comp_PIP

!use constants
use basics
use mathe
use composition

!********************************** VARIABLES *********************************
implicit none
!file parameter
  character(len=11), parameter:: elementfile='elementfile'
  character(len=10), parameter:: radienfile='radienfile'
  character(len=10), parameter:: gridfile='gridfile'
  character(len=11), parameter:: outfile='outcompfile'
  character(len=80) fstr

!material parameter (some are also defined in mod_zss.f90)
  integer ordz, anzele
  double precision atmass

!state variables
  double precision, dimension(:),allocatable,target::alpha,spin
  double precision, pointer::temp,alpha_e,rho
  double precision nheavy
  double precision g_j
  external g_j

!parameter for density/temperature loop and grid
  double precision temp_lb, temp_ub, rho_lb, rho_ub
  double precision dens_step, temp_step
  double precision, dimension(:),allocatable,target::temp_grid
  double precision, dimension(:,:),allocatable,target::dens_grid,alpha_grid
  double precision, dimension(:,:,:),allocatable::result_grid
  integer n_dens,n_temp, t_count, dens_count
  logical lg_scale_dens,lg_scale_temp

!variables for electron density loop
  double precision ne_lb, ne_step
  logical lg_scale_ne

!variables for spline-interpolation
  double precision, dimension(:),pointer::x,y
  double precision, dimension(:), allocatable::z
  double precision, dimension(:,:), allocatable:: coeff
  integer pos_up, pos_down, line_up, line_down, number

!variables for instabilities
  logical:: instab
  integer:: pos_left, pos_right, pos_middle

!variables for output
  character(len=8), dimension(:), allocatable:: str_alph
  integer, parameter:: i0=iachar('0')
  logical default_output

!helping variables
  double precision, dimension(:),allocatable,target::n
  double precision, dimension(:),allocatable::beta_k,n_old
  double precision, dimension(:,:),allocatable,target:: calc_alphas

  double precision, dimension(:),allocatable::abserr,relerr
  double precision fehler, alph_alt, alph_neu

  double precision kappaquad,eta,diff_e, divide
  double precision nesi_step_alt

  double precision, dimension(:),allocatable::help_od
  double precision, dimension(:,:),allocatable::help_td
  double precision help, help_tl
  double precision, pointer:: v_x, v_y

  integer line, column, material, i, j, k, count, stelle1, drop_count
  logical tab_stop, start_ok, write_ok, halt, nohalt, drop, control
  character(len=200) text


!********************************** START ************************************
control=.false.
!control=.true.
control1=.false.
!control1=.true.

!material and density/temperature grid
  lg_scale_dens=.false.
  lg_scale_temp=.false.

  if (control) print*, 'reading gridfile'
  open(10,file=gridfile,status='old',access='sequential')
  read(10,*), text, text
  if (text=='y') then
     default_output=.true.
  else
     default_output=.false.
  end if
  read(10,*), text, text
  if (text=='y') then
     tab_hs=.true.
  else
     tab_hs=.false.
  end if
  read(10,*)
  read(10,*), ordz
  read(10,*), rho_lb,rho_ub,n_dens
  read(10,*), temp_lb,temp_ub,n_temp
  close(10, status='keep')

  if (control) print*, 'determination of density/temperature grid'
  allocate(dens_grid(1:n_dens,2),temp_grid(1:n_temp))
  dens_grid(1,1)=rho_lb
  dens_grid(n_dens,1)=rho_ub
  dens_step=(rho_ub-rho_lb)
  if (dens_step/=0.0d0) then
     if (n_dens==1) then
        print*, 'number of density points is set to 2'
        n_dens=2
     end if
     dens_step=dens_step/dble(n_dens-1)
  end if

  if (rho_lb/=0.0d0) then
     help=rho_ub/rho_lb
     if (dlog10(help)>1) then
        lg_scale_dens=.true.
        dens_step=help**(1.0d0/dble(n_dens-1))
     end if
  end if

  if (lg_scale_dens) then
     do i=2,n_dens-1
        dens_grid(i,1)=dens_grid(i-1,1)*dens_step
     end do
  else
     do i=2,n_dens-1
        dens_grid(i,1)=dens_grid(i-1,1)+dens_step
     end do
  end if

  temp_grid(1)=temp_lb
  temp_grid(n_temp)=temp_ub
  temp_step=(temp_ub-temp_lb)
  if (temp_step/=0.0d0) then
     if (n_temp==1) then
        print*, 'number of temperature points is set to 2'
        n_temp=2
     end if
     temp_step=temp_step/dble(n_temp-1)
  end if

  if (temp_lb/=0.0d0) then
     help=temp_ub/temp_lb
     if (dlog10(help)>1) then
        lg_scale_temp=.true.
        temp_step=help**(1.0d0/dble(n_temp-1))
     end if
  end if

  if (lg_scale_temp) then
     do i=2,n_temp-1
        temp_grid(i)=temp_grid(i-1)*temp_step
     end do
  else
     do i=2,n_temp-1
        temp_grid(i)=temp_grid(i-1)+temp_step
     end do
  end if


!input of material parameters
  if (control) print*, 'reading elementfile'
  open(10,file=elementfile, status='old', access='sequential')
  allocate(help_od(1:100))
  read(10,*) text,anzele
  i=0
  line=1
  do while (i/=ordz .and. line<=anzele)
     read(10,*) text,i,atmass,metomi,pol,maxion,help_od(1:maxion)
     line=line+1
  end do
  close(10, status='keep')

  if (line>anzele) then
     write(*,900) ordz
     stop
  else
     rpol=(pol/ordz**(1.0d0/3.0d0)/2.0d0)**0.25d0      ! Mittelman/Watson
     if (ordz==1) rpol=1.4565d0
     allocate(energy(1:maxion))
     energy(1:maxion)=help_od(1:maxion)/13.602d0
     dens_grid(:,2)=dens_grid(:,1)*1.0d-3/atmass/atommasse
     deallocate(help_od)
     if (.not. default_output) then
        allocate(result_grid(1:n_dens,-1:maxion,1:n_temp))
        result_grid=0.0d0
     end if
  end if

900 format&
   &('Sorry, parameter for element ',I2,' could not be found in elementfile.'/&
   & 'Please check ordering number in gridfile or modify elementfile!')

!with hard sphere interaction?
  if (tab_hs) then 
     allocate(radius(0:maxion),eta0(0:maxion))
     if (control) print*, 'reading radienfile'
     open(10,file=radienfile, status='old', access='sequential')

     line=line-1
     do j=1,line-1
        read(10,*) text,i
     end do
     read(10,*) text,i,radius(0:maxion)
     close(10, status='keep')

     if (maxval(radius)==0.0d0) then
        write(*,910) ordz
        tab_stop=affirm('Continue without hard sphere interaction?')
        if (tab_stop) then
           tab_hs=.false.
           deallocate(radius)
        else
           stop
        end if
     else
        eta0=4.0d0*pi/3.0d0*radius**3
     end if
  end if

910 format&
   &('Radii for element ',I2,' are given as 0 in radienfile.')


!other parameter
  allocate(spin(0:maxion),spinparameter(1:maxion))
!  spin=2.0d0
!  do i=mod(ordz,2),maxion,2
!     spin(i)=1.0d0
!  end do
! *********Calculation of the spinparamters *********
   if (control) print* ,'Calculation of spinparameters:'
   do i=0,maxion,1
      spin(i)=g_j(ordz,i)
      if (control) print*,'g_z',i,' = ',spin(i)
   end do
   if (control) print*, 'Done!'
  spinparameter(1:maxion)=spin(0:maxion-1)/spin(1:maxion)

  allocate(str_alph(0:maxion))
  do i=0,maxion
     str_alph(i)='alpha_'//achar(i0+i)
  end do


!open outputfile
  open(10,file=outfile, status='replace', access='sequential')
  if (default_output) then
     write(10,90) 'y'
  else
     write(10,90) 'n'
  end if
90 format('#default_form ',A2)
  write(10,100) ordz, n_temp, n_dens
100 format('#element ',I2,/&
          &'#molecules n'/&
          &'#n_T = ',I4,/&
          &'#n_rho = ',I4)

if (control) print*, 'ready with preparation, starting of calculation'

!******************************************************************************
!*********************** CALCULATION OF COMPOSITION ***************************
!******************************************************************************

!allocations & first initializations
  allocate(calc_alphas(1000,-2:maxion))
  rho => calc_alphas(1,-2)
  allocate(n(-1:maxion),n_old(-1:maxion))
  allocate(i_array(0:maxion),beta_k(1:maxion))
  i_array=(/ (dble(i),i=0,maxion) /)
  allocate(abserr(-1:maxion),relerr(-1:maxion))

  lg_scale_ne=.true.
  ne_lb=rho_lb*1.0d3/atmass/atommasse
  ne_lb=min(ne_lb,1.0d24) !makes an sufficiant small initial value 
  ne_lb=ne_lb*abohr**3
  ne_step=1.1d0
  ne => n(-1)

open(20, status="replace")
!******************************************************************************
!temperature loop
!******************************************************************************
  do t_count=1,n_temp
     if (control) print*, 'temperature loop'

     !initializing
     start_ok=.true.
     write_ok=.false.
     drop_count=0
     if (ne_step/=1.1d0) ne_step=1.1d0
     
     calc_alphas=0.0d0
     dens_count=0
     n=0.0d0
     n(-1)=ne_lb

     Z_min=0
     Z_max=maxion

     temp => temp_grid(t_count)
     kbt=kbat*temp
     thermlambda=2.0d0*pi*planckquerat/(dsqrt(2.0d0*pi*kbt*emasseat))
     help_tl=thermlambda**3/2.0d0


     if (control) print*, 'temperature loop - ready with initializing'
    
     !*************************************************************************
     !density loop
     !*************************************************************************
     do while (rho<=rho_ub .or. start_ok .or. dens_count<4)
        if (control) print*, 'density loop'
        
        !initializing
        dens_count=dens_count+1
        nullify(rho)
        rho => calc_alphas(dens_count,-2)
        kappaquad=equadratat/epsilon0at/kbt/help*fermdach(help*n(-1), -0.5)
        kappa=dsqrt(kappaquad)   !inverse screening radius
        mue_e_id=kbt*ferinv(help_tl*n(-1), 0.5) !id. part chem. pot. of e^{-}
        call virkoff(bca)        !virial coefficient

        !new Z_min/Z_max
        if (.not. start_ok .and. maxion>1) then 
           if (control) print*, 'new Z_min/Z_max'
           Z_min=0
           do while(calc_alphas(dens_count-1,Z_min)<1.0d-12)
              Z_min=Z_min+1
           end do
           if (calc_alphas(dens_count-1,Z_min)>1.0d-6 .and. Z_min/=0) &
                Z_min=Z_min-1
           Z_max=maxion
           do while(calc_alphas(dens_count-1,Z_max)<1.0d-12)
              Z_max=Z_max-1
           end do
           if (calc_alphas(dens_count-1,Z_max)>1.0d-6 .and. Z_max/=maxion)&
                Z_max=Z_max+1
        end if
        minion=max(1,Z_min)
        
        !determination of n
        if (Z_max==1 .and. .not. tab_hs) then
           n(1)=n(-1)
           n(0)=0.0d0
           call mk_k(n,beta_k)   !returns exponent beta(mue_e+delta_mue_i)
           call mk_n0(beta_k,n)  !returns number of atoms
        else
           !initial n
           if (control) print*, 'initial n'
           if (start_ok) then
              !very first n
              beta_k=mue_e_id+energy
              beta_k=beta_k/kbt
              call mk_n(beta_k,n)
              n(0)=1.0d-200
              rho=sum(n(0:maxion))/abohr**3/1.0d3*atmass*atommasse
           else
              !finding maximum in old n and putting rest of new ne on it
              stelle1=max(Z_min,1)
              do i=Z_min+1,Z_max
                 if (n(stelle1)<=n(i)) stelle1=i
              end do
              diff_e=ne-sum(i_array(Z_min:Z_max)*n(Z_min:Z_max))
              n(stelle1)=n(stelle1)+diff_e/dble(stelle1)
           end if
           
           !iteration for n
           if (control) print*, 'iteration for n'
           nohalt=.true.
           drop=.false.
           count=1
           do while (nohalt)
              n_old=n
              
              call mk_k(n,beta_k) 
              if (Z_max==1) then
                 call mk_n0_hs(beta_k,n)
              else
                 call mk_n(beta_k,n)
              end if

              !termination condition
              fehler=1.0d-5
              abserr=dabs((n-n_old))
              relerr=fehler*n
              if (maxval(abserr-relerr)<=0.0d0) nohalt=.false.
              
              if (nohalt) then
                 allocate(help_od(minion:Z_max))
                 help_od=i_array(minion:Z_max)*n_old(minion:Z_max)
                 alph_alt=sum(help_od)/sum(n_old(Z_min:Z_max))
                 help_od=i_array(minion:Z_max)*n(minion:Z_max)
                 alph_neu=sum(help_od)/sum(n(Z_min:Z_max))
                 if (dabs(alph_alt-alph_neu)<1.0d-8) nohalt=.false.
                 deallocate(help_od)
              end if
              
              !accelaration of iteration
              if (nohalt) then
                 divide=0.5
                 if (count>4000) call random_number(divide)
                 if (count>=2000 .and. count<5000) &
                      n=divide*n+(1.0d0-divide)*n_old
                 if (count>=5000 .and. write_ok) then 
                    nohalt=.false.
                    if (dabs(alph_alt-alph_neu)>5.0d-6*(alph_alt+alph_neu))&
                         drop=.true.
                 end if
              end if

              !test for packing fraction
              if (nohalt .and. tab_hs) then
                 eta=sum(eta0(Z_min:Z_max)*n(Z_min:Z_max))
                 do while (eta>=0.99d0) 
                    i=1
                    do j=2,maxion-1
                       if (n(j)>n(i)) i=j
                    end do
                    j=i+1
                    do while(radius(i)==radius(j) .and. j<maxion)
                       j=j+1
                    end do
                    help=sum(0.1d0*n(1:j-1)*i_array(1:j-1))
                    n(0:j-1)=0.9d0*n(0:j-1)
                    n(j)=n(j)+help/i_array(j)
                    eta=sum(eta0*n(0:maxion))
                 end do
                 if (eta>=1) then 
                    print*, 'Calculation with hard sphere contribution &
                            & is not possible'
                    print*, 'anymore. Program will be continued without hs.'
                    tab_hs=.false.
                 end if
              end if

              count=count+1
           end do   !end of iteration for n
           if (control) print*, 'end of iteration'
        end if      !end of determination of n
        if (control) print*, 'end of determination of n'
           
        
        if (.not. drop) then
           !calculation of calc_alpha(count_dens)
           nheavy=sum(n(Z_min:Z_max))
           calc_alphas(dens_count,-1:maxion)=n(-1:maxion)/nheavy
           !prevention of jumps disturbing spline interpolation
           if (dens_count >2) then
              allocate(help_od(Z_min:Z_max))
              help_od(:)=calc_alphas(dens_count-1,Z_min:Z_max)
              help_od(:)=help_od(:)/calc_alphas(dens_count,Z_min:Z_max)
              help_od(:)=dabs(dlog10(help_od(:)))
              help=dlog10(calc_alphas(dens_count-1,-2)&
                   /(nheavy/abohr**3/1.0d3*atmass*atommasse))
              if (maxval(help_od)>2.3d0 .and. dabs(help)<0.3d0) drop=.true.
              deallocate(help_od)
           end if
        end if

        if (drop) then
           dens_count=dens_count-1
          ! help=sum(n(Z_min:Z_max))
          ! if (calc_alphas(dens_count-1,-2)>1.0d0) &
          !      write(30,*) help/abohr**3/1.0d3*atmass*atommasse,n(1)/help
           drop_count=drop_count+1
        else
           rho=nheavy/abohr**3/1.0d3*atmass*atommasse 
           if (rho<rho_lb) then
              start_ok=.false.
              write_ok=.true.
              calc_alphas(1,:)=calc_alphas(dens_count,:)
              dens_count=1
           end if
           drop_count=0
        end if

        !new density
        if (control) print*, 'new density'        
        !test for ne_step
        if (drop_count>1) then
           drop_count=0
           if (ne_step>=1.02) then
              ne_step=ne_step-1.0d0
              ne_step=ne_step/2.0d0+1.0d0
           end if
        end if
        if (lg_scale_ne) then
           if (start_ok) then 
              ne=ne/5.0d0/ne_step
           else 
              ne=ne*ne_step
           end if
        else
           if (start_ok) then 
              ne=ne-5.0d0*ne_step
           else 
              ne=ne+ne_step
           end if
        end if

        if (control) print*, 'end density loop'        
     end do

     !*************************************************************************
     !end density loop
     !*************************************************************************

     !control file
     if (control) print*, 'write to fort.20'
     write(20,110) int(temp_grid(t_count))
     write(20,fmt='(A24)',advance="no") '#    rho       alpha_e  '
     do i=0,maxion
        write(20,131,advance="no") str_alph(i)
     end do
     write(20,*)
     do i=1,dens_count
        do j=-2,maxion
           write(20,141,advance="no") calc_alphas(i,j)
        end do
        write(20,*)
     end do
     write(20,*), '&'

     !looking for instability / determination of instability positions
     if (control) print*, 'instability region'

     instab=.false.
     pos_left=0   !position in calc_alphas
     pos_right=0  !position in calc_alphas
     pos_middle=1 !position in dens_grid
     x=>calc_alphas(1:dens_count,-2)

     j=1
     do while ( x(j)<x(j+1) .and. j<dens_count)
        j=j+1
     end do
     pos_left=j
     if (pos_left/=dens_count) instab=.true.

     if (instab) then
        pos_right=pos_left+1
        do while (x(pos_right)>x(pos_right+1))
           pos_right=pos_right+1
        end do

        help=(x(pos_left)+x(pos_right))/2.0d0
        do while (dens_grid(pos_middle,1)<help)
           pos_middle=pos_middle+1 
        end do
        pos_middle=pos_middle-1 
        do while (x(pos_left)>help)
           pos_left=pos_left-1 
        end do
        pos_left=pos_left+1 
        do while (x(pos_right)<help)
           pos_right=pos_right+1 
        end do
        pos_right=pos_right-1 
     else
        pos_middle=n_dens
     end if
     nullify(x)

     !spline interpolation for all alphas 
     if (control) print*, 'spline interpolation'

     Z_min=0
     do while(maxval(calc_alphas(:,Z_min))<1.0d-6 .and. Z_min<Z_max)
        Z_min=Z_min+1
     end do
     Z_max=maxion
     do while(maxval(calc_alphas(:,Z_max))<1.0d-6 .and. Z_max>Z_min)
        Z_max=Z_max-1
     end do

     allocate(alpha_grid(n_dens,-1:Z_max))
     alpha_grid=0.0d0

     x=>calc_alphas(1:pos_left,-2)
     number=pos_left
     allocate(coeff(number,5),z(1:number))
     do j=-1,Z_max
        if (j==-1 .or. j>=Z_min) then
           z=calc_alphas(1:number,j)
           z=dlog10(z)
           call cubspl(x,z,number,coeff)
           do k=1,pos_middle
              alpha_grid(k,j)=spl(dens_grid(k,1),coeff,number)
              alpha_grid(k,j)=10**alpha_grid(k,j)
           end do
           nullify(y)
        end if
     end do
     deallocate(coeff,z)
     nullify(x)
        
     if (instab) then
        x =>calc_alphas(pos_right:dens_count,-2)
        number=size(x,1)
        allocate(coeff(1:number,5),z(1:number))
        do j=-1,Z_max
           if (j==-1 .or. j>=Z_min) then
              z= calc_alphas(pos_right:dens_count,j)
              z=dlog10(z)
              call cubspl(x,z,number,coeff)
              do k=pos_middle+1,n_dens
                 alpha_grid(k,j)=spl(dens_grid(k,1),coeff,number)
                 alpha_grid(k,j)=10**alpha_grid(k,j)
              end do
           end if
        end do
        deallocate(coeff,z)
        nullify(x)
     end if

     !*************************************************************************
     !output in outcompfile, if default_output
     !*************************************************************************
     where(calc_alphas<1.0d-99) calc_alphas=0
     where(alpha_grid<1.0d-99) alpha_grid=0

     if (default_output) then
        if(control) print*, 'output'

        write(10,101)
101     format('#-------------------------')

        write(10,110) int(temp_grid(t_count))
110     format('#T = ',I8,' K')
        number=0
        if (instab) number=pos_right-pos_left+1
        write(10,120) pos_middle,number
120     format('#instability',2I4)
        write(10,125) Z_min, Z_max
125     format('#ions',I2,' to', I3)

        if (instab) then
           allocate(help_td(pos_left:pos_right,1:3))
           help_td(pos_left:pos_right,1)=calc_alphas(pos_left:pos_right,-2)
           help_td(pos_left:pos_right,2)&
                =help_td(pos_left:pos_right,1)/atmass/atommasse*1.0d-3
           help_td(pos_left:pos_right,3)=calc_alphas(pos_left:pos_right,-1)
        end if

        number=Z_max-Z_min+1

        write(10,130,advance="no") 
        do i=Z_min,Z_max
           write(10,131,advance="no") str_alph(i)
        end do
        write(10,*)
        write(10,132) 
        do i=1,pos_middle
           write(10,140,advance="no") dens_grid(i,:),alpha_grid(i,-1)
           do j=Z_min,Z_max
              write(10,141,advance="no") alpha_grid(i,j)
           end do
           write(10,*)
        end do
        if (instab) then
           do i=pos_left,pos_right
              write(10,140,advance="no") help_td(i,1:3)
           do j=Z_min,Z_max
              write(10,141,advance="no") calc_alphas(i,j)
           end do
           write(10,*)
           end do
           deallocate(help_td)
           do i=pos_middle+1,n_dens
              write(10,140,advance="no") dens_grid(i,:),alpha_grid(i,-1)
              do j=Z_min,Z_max
                 write(10,141,advance="no") alpha_grid(i,j)
              end do
              write(10,*)
           end do
        end if

130 format('#  rho      n_heavy    alpha_e') 
131 format(A12) 
132 format('# g/ccm      1/ccm')

140 format(E9.3,2E11.3) 
141 format(E12.5) 

     else
        !write to result_grid
        result_grid(1:n_dens,-1,t_count)=alpha_grid(1:n_dens, -1)
        do i=Z_min,Z_max
           result_grid(1:n_dens,i,t_count)=alpha_grid(1:n_dens,i)
        end do
     end if

     !deallocations
     deallocate(alpha_grid)
     if (tab_hs) deallocate(eta0,radius)
     nullify(temp)

  end do
  !****************************************************************************
  !end temperature loop
  !****************************************************************************


  !****************************************************************************
  !output in outcompfile, if not default_output
  !****************************************************************************

  if (.not. default_output) then
     if(control) print*, 'output'
    
     do dens_count=1,n_dens
        write(10,101)
        
        write(10,150) dens_grid(dens_count,1)
150     format('#rho = ',E9.3,' g/ccm')

        Z_min=0
        do while(maxval(result_grid(dens_count,Z_min,:))<1.0d-6 &
             .and. Z_min<maxion)
           Z_min=Z_min+1
        end do
        Z_max=maxion
        do while(maxval(result_grid(dens_count,Z_max,:))<1.0d-6 &
             .and. Z_max>max(1,Z_min))
           Z_max=Z_max-1
        end do
        write(10,125) Z_min, Z_max

        write(10,160,advance="no") 
        do i=Z_min,Z_max
           write(10,161,advance="no") str_alph(i)
        end do
        write(10,*)
        write(10,162) 
        do j=1,n_temp
           write(10,170,advance="no") int(temp_grid(j)),&
                result_grid(dens_count,-1,j)
           do i=Z_min,Z_max
              write(10,171,advance="no") result_grid(dens_count,i,j)
           end do
           write(10,*)
        end do

160 format('#   T      alpha_e') 
161 format(A12) 
162 format('#   K    ')

170 format(I8,E11.3) 
171 format(E12.5) 

     end do
     deallocate(result_grid,dens_grid,temp_grid)
  end if

  close(10, status='keep')
print*, 'ready with composition :-)'  
end program

