
!******************************************************************************
!******************************************************************************
!******************************************************************************

module fermis
use constants
use mathe

contains

!**************************** Fermi-Dirac functions ***************************

double precision function ferm(x, ny)

!ferm(x,ny)=1/Gamma(ny+1)*Int(t^ny/(1+exp(t-x)))dt
!with ny=-0.5, 0.5, 1.0, 1.5, 2.0, 3.0 und 4.0.
!
!For ny=2.5, 3.5, 4.5 ferm is given by an approximation: 
!->boltz(x, ny) for x<2.0 and ->herm(x, ny) or ->somf(x, ny) else.

  implicit none

!in- and output 
  double precision, intent (in) :: x          
  real, intent (in) :: ny          

!helping variables
  double precision:: x1, dferdr        !dferdr is CERN routine
  integer:: i, n
  integer, parameter:: k=1d4
  double precision, dimension(k):: summand
  double precision ug, og
                   
 
!Fermi function for |ny|=1.5 or |ny|=0.5 
  if (abs(ny)==0.5d0 .or. abs(ny)==1.5d0) then
     n=int(2*ny)
     ferm=dferdr( x, n)
     if (ny==-1.5) ferm=1/(2*dsqrt(pi))*ferm
     if (ny==-0.5) ferm=1/dsqrt(pi)*ferm
     if (ny==0.5) ferm= 2/dsqrt(pi)*ferm
     if (ny==1.5) ferm= 4/(3*dsqrt(pi))*ferm
  end if


!Fermi function for ny=1..4
  if (int(ny)==ny) then
     if (x<=0) then 
        x1=-dexp(x)
        do i=1,k
           summand(i)=(x1)**i/i**(ny+1)
        end do
        ferm=-sum(summand)
     else
        x1=-dexp(-x)
        do i=1,k
           summand(i)=(x1)**i/i**(ny+1)
        end do
        if (ny==1) ferm=1.0d0/2.0d0*x**2+pi**2/6.0d0+sum(summand)
        if (ny==2) ferm=1.0d0/6.0d0*x**3+pi**2/6.0d0*x-sum(summand)
        if (ny==3) ferm=&
          &1.0d0/24.0d0*x**4+pi**2/12.0d0*x**2+7.0d0*pi**4/360.0d0+sum(summand)
        if (ny==4) ferm=&
           &1.0d0/120.0d0*x**5+pi**2/36.0d0*x**3+7.0d0*pi**4/360*x-sum(summand)
     end if
  end if

!Fermi functionen for ny=2.5, ny=3.5, ny=4.5
  if (ny==2.5 .or. ny==3.5 .or. ny==4.5) then
     if (ny==2.5 .or. ny==3.5) then
        ug=-2.0d0
        og=8.0d0
     end if

     if (ny==4.5) then
        ug=-2.0d0
        og=10.0d0
     end if

     if (x<ug) ferm=boltz(x, ny)
     if (x>ug .and. x<og) ferm=herm(x, ny)
     if (x>og) ferm=somf(x, ny)
  end if

end function ferm


!********************************

double precision function ferinv(x, ny)

!inverse function ->ferm(x, 0.5)
!Ref: R.Zimmermann: "Many Particle Theorie of Highly Excited Semiconductors"
!                   Teubner-Texte zur Physik, Band 18, S.151
!                   Teubner Verlagsgesellschaft, Leipzig, 1988

  implicit none

!variables
  double precision, intent(in)::x
  real, intent(in)::ny

!warning and calculation
  if (ny/=0.5) then 
     print*,'ERROR: The inverse Fermi-Dirac function is not coded for ny/=0.5'
     stop
  else
     if (x<5.5d0) ferinv=(dlog(x)+0.3536d0*x-0.00495d0*x**2+0.000125*x**3)
     if (x>=5.5d0) ferinv=1.209d0*x**(2.0d0/3.0d0)-0.6803d0*x**(-2.0d0/3.0d0)&
                         &-0.85d0*x**(-2)
  end if

end function

!********************************

double precision function fermdach(x,ny)

!calculation of ferm(ferinv(x,0.5), ny) for ny=-0.5 and -1.5
!Ref: R.Zimmermann: "Many Particle Theorie of Highly Excited Semiconductors"
!                   Teubner-Texte zur Physik, Band 18, S.151
!                   Teubner Verlagsgesellschaft, Leipzig, 1988

  implicit none

!variables
  double precision, intent(in)::x
  real, intent(in)::ny

!warning
  if (ny/=-0.5 .and. ny/=-1.5) then
     print*, 'ERROR: fermdach only coded for ny=-0.5 or ny=-1.5!'
     stop
  end if

!Berechnung
 if (ny==-0.5) then
    if (x<5.5) then 
       fermdach=x/(1.0d0+0.3536d0*x-0.0099*x**2+0.000375*x**3)
    else
       fermdach=x**(1.0d0/3.0d0)/(0.806d0+0.4535d0*x**(-4.0d0/3.0d0)&
               &+1.7d0*x**(-8.0d0/3.0d0))
    end if
 end if

 if (ny==-1.5) then
    if (x<5.5) then 
       fermdach=x+0.1768d0*x**2-0.0033d0*x**3+0.000094*x**4
    else
       fermdach=0.4836d0*x**(5.0d0/3.0d0)+1.3606*x**(1.0d0/3.0d0)-1.7d0*x**(-1)
    end if
 end if

end function

!************** BOLTZ, HERM & SOMF *********

!used for approximations of Fermi functions, see ->ferm
!Ref: H. Reinholz, priv. communication

double precision function boltz(x, ny)

  implicit none

!variables
  double precision, intent(in):: x
  real, intent(in)::ny
  double precision hilf

!calculation
  hilf=dexp(x)
  boltz=hilf*(1.0d0-hilf*(1.0d0/2.0d0**(ny+1)-hilf/3.0d0**(ny+1)))

end function boltz

!******************************
double precision function herm(x, ny)

  implicit none

!variables
  double precision, intent(in):: x
  real, intent(in)::ny
  double precision hilf
  double precision, dimension(32):: a
  double precision, dimension(32,2), parameter::q=reshape(&
       (/4.4489365833d-2,2.3452610952d-1,5.7688462930d-1,1.0724487538d0,&
         1.7224087764d0,2.5283367064d0,3.4922132730d0,4.6164567697d0,&
         5.9039585042d0,7.3581267332d0,8.9829409242d0,10.783018632d0,&
         12.763697987d0,14.931139755d0,17.292454337d0,19.855860940d0,&
         22.630889013d0,25.628636022d0,28.862101816d0,32.346629154d0,&
         36.100494806d0,40.145719771d0,44.509207996d0,49.224394987d0,&
         54.333721333d0,59.892509162d0,65.975377288d0,72.687628091d0,&
         80.187446978d0,88.735340418d0,98.829542868d0,1.117513981d2,&
         1.1418710577d-1,2.6606521690d-1,4.1879313732d-1,5.7253284650d-1,&
         7.2764878838d-1,8.8453671933d-1,1.0436188759d0,1.2053492741d0,&
         1.3702213385d0,1.5387772565d0,1.7116193527d0,1.8894240634d0,&
         2.0729593401d0,2.2631066339d0,2.4608890725d0,2.6675081263d0,&
         2.8843920929d0,3.1132613268d0,3.3562176924d0,3.6158698562d0,&
         3.8955130451d0,4.1993941046d0,4.5331149786d0,4.9042702879d0,&
         5.3235009711d0,5.8063332139d0,6.3766146735d0,7.0735265803d0,&
         7.9676935095d0,9.2050403307d0,11.163013090d0,15.390180401d0/),&
         (/32,2/))
  integer i

!calculation
  hilf=dexp(-x)
  a(1:32)=q(1:32,2)*dexp(-q(1:32,1))*(1.0d0-dexp(-q(1:32,1)))&
          &/(hilf+dexp(-q(1:32,1)))*q(1:32, 1)**ny
  herm=(1.0d0+sum(a)/halffaculty(ny))/(1.0d0+hilf)

end function herm

!******************************
double precision function somf(x, ny)

  implicit none

!variables
  double precision, intent(in):: x
  real, intent(in)::ny
  double precision hilf

!calculation
  hilf=pi**2*ny*(ny+1.0)/(6.0d0*x**2)
  hilf=hilf*(1.0d0+7.0d0*pi**2*(ny-1.0)*(ny-2.0)/(6.0d1*x**2))
  
  somf=(1.0d0+hilf)*x**(ny+1)/halffaculty(ny+1.0)

end function somf

end module fermis
