!******************************************************************************
!******************************************************************************
!******************************************************************************

module mathe

use constants

contains 

!***************************** Integration ************************************

subroutine testgrenze(wo,grenze,err,erg,func)
!expands integration interval, if nessesary

implicit none

!in- and output
  double precision, intent(in):: grenze,err
  double precision, intent(inout):: erg
  character(len=1), intent(in):: wo
  external func

!helping variables
  double precision rest
  double precision bound
  integer stop
  double precision dGauss !CERN integration routine
 
!calculation
  bound=grenze
  rest=0.0d0
  stop=0

  if (wo=='u') rest=dGauss(func,bound/2.0d0,bound,err)
  if (wo=='o') rest=dGauss(func,bound,bound*2.0d0,err)
  erg=erg+rest
  do while(rest/erg>err)
     if (wo=='u') then
        bound=bound/2.0d0
        rest=dGauss(func,bound/2.0d0,bound,err)
     end if
     if (wo=='o') then
        bound=bound*2.0d0
        rest=dGauss(func,bound,bound*2.0d0,err)
     end if
     erg=erg+rest
  end do
end subroutine testgrenze


!**************************** Spline Interpolation ****************************

!->cubspl creates matrix koeff(n,5) which contains coefficients of cubic 
!(natural) spline interpolation
!coulumn 1 and 2 contain x and y
!f(t) [x(i)<=t<x(i+1)] is then calculated via
! f(t)=koeff(i,2)+koeff(i,3)*h(i)+koeff(i,4)*h(i)^2+koeff(i,5)*h(i)^3  with
! h(i)=t-x(i); see function ->spl.

!******* cubspl *************
subroutine cubspl( x, y, n, koeff)

  implicit none

!in- and output
  integer, intent(in)::n
  double precision, dimension(:), intent(in):: x, y
  double precision, dimension(size(x),5), intent(out):: koeff

!helping variables / copies
  double precision, dimension(0:n-1)::x1
  double precision, dimension(0:n-1,1)::y1
  double precision, dimension(n-1,1)::a, b, c, d
  x1(0:n-1)=x(1:n)
  y1(0:n-1,1)=y(1:n)

!calculation
  call dcspln(n-1,x1,1,y1,n-1,0,a,b,c,d) !CERN routine

  koeff(1:n,1)=x(1:n)
  koeff(1:n,2)=y(1:n)
  koeff(1:n-1,3)=b(1:n-1,1)
  koeff(1:n-1,4)=c(1:n-1,1)
  koeff(1:n-1,5)=d(1:n-1,1)

end subroutine

!************* spl ****************
double precision function spl(x, koeff, n)

  implicit none

!variables
  double precision, intent(in):: x
  double precision, dimension(:,:), intent(in):: koeff
  integer, intent(in)::n
  integer i

!warning
  if(x<koeff(1,1) .or. x>koeff(n,1))then
     print*, &
          'ERROR: The Spline interpolation was made for a too small interval!'
     stop
  end if

!calculation
  i=0
  do while (x>=koeff(i+1,1) .and. i<n)
     i=i+1
  end do

  spl=koeff(i,2)+koeff(i,3)*(x-koeff(i,1))+koeff(i,4)*(x-koeff(i,1))**2&
     &+koeff(i,5)*(x-koeff(i,1))**3

end function


!********************************* determinants *******************************

double precision function det(matrix)
!calculation of determinant with splitting in triangle matrices

implicit none

!variables
  double precision, dimension(:,:), intent(in):: matrix
  double precision, dimension(size(matrix,1),size(matrix,2))::a, abs
  double precision, dimension(size(matrix,1))::diag, temp, hilf
  integer i, j
  integer, dimension(1)::imax
  integer n

!warning
  if (size(matrix,1)/=size(matrix,2)) then
     print*, 'ERROR in determinant calculation: Matrix is not quadratic!'
     stop
  end if

!initializing
  n=size(matrix,1)
  a=matrix

!L-R-splitting
  do i=1,n-1
     abs=0.0d0
     abs(i:n,i:n)=dabs(a(i:n,i:n))
     hilf=0.0d0

  !(pivotation in coulumn) 
     hilf(i:n)=abs(i:n,i)
     imax=maxloc(hilf)
     if (imax(1)>i) then
        temp(1:n)=a(i,1:n)
        a(i,1:n)=a(imax(1),1:n)
        a(imax(1),1:n)=-temp(1:n)
     end if

  !diagonal elements of R
     if (maxval(hilf)<1.0d-30) then
        det=0.0d0
        return
     else
        do j=i+1,n
           a(j,1:n)=a(j,1:n)-a(i,1:n)*(a(j,i)/a(i,i))
        end do
        diag(i)=a(i,i)
     end if
  end do

  diag(n)=a(n,n)

!determinant
  det=product(diag)

end function 



!******************************** Faculties *********************************

integer function faculty(n) 

  implicit none

!variables
  integer, intent(in):: n
  integer i, f

!calculation
  i=0
  faculty=1
  do while(i<n)
     i=i+1
     faculty=i*faculty
  end do

end function

!********************************
real function halffaculty(x)

  implicit none

!variables
  real, intent(in):: x
  integer i

!warning
  if (2*x/=int(2*x)) then
     print*, 'ERROR: faculties only coded for n and n/2!'
     return
  end if

!calculation
  i=1
  halffaculty=dsqrt(pi)/2.0d0
  do while(i/2.0<x)
     i=i+2
     halffaculty=i/2.0*halffaculty
  end do

end function
!******************************************************************************

end module

