
!******************************************************************************
!******************************************************************************
!******************************************************************************

module transmod

  use constants
  use mathe
  use fermis

  implicit none
  save

!variables
  !material parameter
    double precision pol, rpol
  !state parameter
    double precision kbt, muekt, thermlambda, dd, ne, nesi, nat, nmo, dist
    double precision theta, gamma         
    double precision charge
    double precision,pointer::temp
  !variables for potentials
    double precision, dimension (:,:), allocatable:: coeffpot
    double precision radius !Cut-Off for Ashcroft empty core potential
    character(len=2) pottype
    integer npot
    logical tabp,screened_pot
  !variables for correlation functions
    double precision rd, rdei, rdee, rdea, kappa
    double precision, dimension (:,:), allocatable:: coeff,coeffe,coeffa,coeffs
    double precision, dimension (:,:), allocatable:: coeffstr
    integer, dimension(:), allocatable:: ntrq_arr
    integer ntrq, ntrqe, ntrqa, nsk,nstr
    integer typ, typs, typbn
    integer dim,dim1,dim2
    logical tabmo
    character(len=1) typ_ea, typ_ei, typ_ee, typ_em
  !set of calculation
    logical, dimension(4)::do_trans
  !functions from outside libraries and variables there
    double precision, external:: d01ahf                   !NAG integration
    double precision lb, ub, neval, esterr
    integer err

contains

!************************ Parameters ***********************************

subroutine transpar

!calculates plasma paramter. Names are used as usual; dd is prefactor of 
!correlation function D_nm, dist (=distance) is average distance between 
!electrons (therefore, gamma=gamma(Z=1)). Calculations are in atmomic units, 
!(4*pi*epsilon0) is neglected in definitions.


  nesi=ne/abohr**3
  muekt=ferinv(thermlambda**3.0d0*ne/2.0d0, 0.5)
  dist=(3.0d0/(4.0d0*pi*ne))**(1.0d0/3.0d0)

  theta=kbt/(3.0d0*pi**2*ne)**(2.0d0/3.0d0)    
  gamma=equadratat/(kbt*dist)

  dd=16.0d0/3.0d0*dsqrt(pi)/kbt**1.5d0  

end subroutine


!***************************** Potentials ******************************

!********* dielectric function ************
subroutine eps(q,omega,epsilon)

!calculates the real and the imagine part of the dielectric screening function 
!with local field correction of Ichimaru and Utsumi.
!
!Ref: Phys.Rev.B _24_ (1981) 7385-7388

!variables
  double precision, intent(in)::q, omega
  double precision,dimension(2),intent(out)::epsilon
  double precision,dimension(2):: polRPA,hilf,hilf1, quotient 
  double precision v
  
!calculation  
  v=4.0d0*pi*equadratat/q**2
  polRPA(1)=lindhardre(q,omega)
  polRPA(2)=lindhardim(q,omega)
  hilf=v*polRPA

  hilf1=locfieldcorr(q)*hilf
  hilf1(1)=1.0d0+hilf1(1)

  quotient(1)=(hilf(1)*hilf1(1)+hilf(2)*hilf1(2))/sum(hilf1**2)
  quotient(2)=(hilf1(1)*hilf(2)-hilf(1)*hilf1(2))/sum(hilf1**2)
  epsilon=-quotient
  epsilon(1)=1.0d0+epsilon(1)

end subroutine

!***** static dielectric function ******
double precision function  epsstat(q)

!calculates the real part of the dielectric screening function for omega=0
!with local field correction of Ichimaru and Utsumi.
!
!Ref: Phys.Rev.B _24_ (1981) 7385-7388

!variables
  double precision, intent(in)::q
  double precision:: vpi, quotient 
  double precision v, hilf
  
!calculation
!  v=4.0d0*pi*equadratat/q**2
!  vpi=lindhardre(q,0.0d0)*v
!  hilf=1.0d0+locfieldcorr(q)*vpi

  vpi=-(kappa/q)**2
  hilf=1.0d0+locfieldcorr(q)*vpi

  quotient=vpi/hilf
  epsstat=1.0d0-quotient

end function

!************ real part eps ***********
double precision function lindhardre(q,omega)
!real part of RPA-polaristion function

!variables
  double precision, intent(in)::q, omega
  double precision grossq, omega1,err, hmm
  common/hier/grossq, omega1

!calculation
  grossq=q/dsqrt(kbt)
  omega1=omega
  call dadapt(integrant,1.0d-3,1.0d1,1,1.0d-3,1.0d-6,hmm,err)
  lindhardre=0.5d0/pi**2*0.5d0/grossq*dsqrt(kbt)*hmm

!needed functions
contains
double precision function integrant(s)
  double precision, intent(in)::s
  double precision  grossq, omega
  common/hier/grossq, omega
  double precision factor1, factor2, factor3, hilf

  factor1=s
  factor2=1.0d0/(dexp(s**2-muekt)+1.0d0)
  hilf=(0.5d0*omega/kbt/grossq)**2
  factor3=dlog(((s-0.5d0*grossq)**2-hilf)/((s+0.5d0*grossq)**2-hilf))

  integrant=factor1*factor2*factor3
end function

end function

!********** imaginary part eps *************
double precision function lindhardim(q,omega)
!imagine part of RPA-polaristion function

!variables
  double precision, intent(in)::q, omega
  double precision grossq
  double precision factor1, factor2, hilf1, hilf2

!calculation
  grossq=q/dsqrt(kbt)

  factor1=kbt/q
  hilf1=1.0d0+dexp(-0.25*(grossq+omega/kbt/grossq)**2+muekt)
  hilf2=1.0d0+dexp(-0.25*(grossq-omega/kbt/grossq)**2+muekt)
  factor2=dlog(hilf1/hilf2)

  lindhardim=0.125d0/pi*factor1*factor2
end function

double precision function lindhard2(q)
!RPA-polaristion function, with Coulomb-potential V(q)=4pi*e^2/q^2 

!variables
  double precision, intent(in)::q
  double precision  hmm, hilf1, hilf2

!calculation
  hmm=dsqrt(kbt/theta)
  hilf1=(4.0d0*hmm**2-q**2)/(4.0d0*hmm*q)
  hilf2=dlog(dabs((2.0d0*hmm+q)/(2.0d0*hmm-q)))
  lindhard2=-2.0d0/pi/q**2*hmm*(1.0d0+hilf1*hilf2)
end function

!**************** LFC *******************
double precision function locfieldcorr(q)

!local field correction by Ichimaru/Utsumi
!Ref: Phys.Rev.B _24_ (1981) 7385-7388

!variables
  double precision, intent(in)::q
  double precision qf, a, b, c
  double precision gamma0, g0, alfa, rs, z
  double precision, external:: dBesI1
  double precision dnachdrs, hilf
  double precision, parameter::&
       b0=6.21814d-2, b1=9.81379d0, b2=2.82224d0, b3=7.36411d-1
  
!calculation
  qf=q/dsqrt(kbt/theta)
  rs=dist

  hilf=dsqrt(rs)
  dnachdrs=-0.5d0*b0 &
          *(7.0d0+14.0d0*b1*hilf+(9.0d0*b2+7.0d0*b1**2)*hilf**2 &
          +(10.0d0*b3+8.0d0*b1*b2)*hilf**3+9.0d0*b1*b3*hilf**4) &
          /hilf**8/(1.0d0+b1*hilf+b2*hilf**2+b3*hilf**3)**2

  alfa=(4.0d0/9.0d0/pi)**(1.0d0/3.0d0)
  gamma0=0.25d0-pi*alfa/24.0d0*rs**5*dnachdrs
  z=4.0d0*dsqrt(alfa*rs/pi)
  g0=0.125d0*(z/dBesI1(z))**2

  a=0.029d0
  b=9.0d0/16.0d0*gamma0-3.0d0/64.0d0*(1.0d0-g0)-16.0d0/15.0d0*a
  c=-0.75d0*gamma0+9.0d0/16.0d0*(1.0d0-g0)-16.0d0/5.0d0*a

  locfieldcorr=a*qf**4+b*qf**2+c&
              +(a*qf**4+(b+8.0d0/3.0d0*a)*qf**2-c)&
              *(4.0d0-qf**2)/4.0d0/qf*dlog(dabs((2.0d0+qf)/(2.0d0-qf)))
end function

!******** different potentials **********
double precision function pot(q)

!variables
  double precision, intent(in)::q
  double precision, dimension(2):: epsilon

!determination
  if (pottype=='SC') pot=-(4.0d0*pi*equadratat)/q**2                !Coulomb
  if (pottype=='DB') pot=-(4.0d0*pi*equadratat)/(kappa**2+q**2)     !Debye
  if (pottype=='AS') pot=-(4.0d0*pi*equadratat)*dcos(q*radius)/q**2 !Ashcroft
  if (pottype=='PP') then                                           !Pseudopot
     if (q<=coeffpot(npot,1) .and. q>=coeffpot(1,1)) then 
        pot=spl(q,coeffpot,npot)    
     else
        if (screened_pot) then
           pot=-(4.0d0*pi*equadratat)/(kappa**2+q**2)   !Debye
        else
           pot=-(4.0d0*pi*equadratat)/q**2              !Coulomb
        end if
     end if
  end if
  if (.not. screened_pot) then                             
     if (pottype=='SC') then 
        epsilon(1)=epsstat(q) !screened by RPA+LFC(IU)
     else
        epsilon(1)=1.0d0+(kappa/q)**2 !screened by RPA
     end if
     pot=pot/epsilon(1)
  end if
end function

!************************ Correlations ***********************************

!calculation of integrants for correlation functions D_{ei}, D_{ee}, D_{ea}. 
!The variable typ determinds different possibilities: 
!      typ=1 : T-matrix
!      typ=2 : Born approximation
!      typ=3 : Fit by Roepke/Redmer and Meyer/Bartoly, resp.
!
!Ref: G.Roepke and R.Redmer, PRA 39 (1989) 907
!     J.R.Meyer and F.J.Bartoli, PRB 23 (1981) 5413

double precision function psdei(k)
!in contrast to =>dei numerical integration, suitable for different potentials

implicit none
!variables
  double precision, intent(in)::k
  double precision qt
  double precision b, phi
  double precision vorf, f
  double precision error,dgauss, hilfphi, ku, ko

!calculation of transport cross sections
  if (typ_ei=='t' .and. k<=coeff(ntrq,1)) then
     if (k<coeff(1,1)) then 
        qt=coeff(1,2)
     else
        qt=spl(k, coeff, ntrq)
     end if
  else
     phi=0.0d0
     call dadapt(intqt,0,2.0d0*k,1,1.0d-4,1.0d-6,phi,error)
     qt=4.0d0*pi*charge/k**4*phi
  end if

!determination of prefactor and f(1-f)
  vorf=dd/(ne*thermlambda**3*pi)
  f=0.0d0
  if (k<dsqrt((5.0d1+muekt)*kbt)) &
       f=(1.0d0+dexp(-(k**2/kbt-muekt)))**(-2)*dexp(-(k**2/kbt-muekt))

!calculation of psdei
  psdei=vorf*k*k**2*(k**2/kbt)**(dim+1)*f*qt
 
end function

!**** integrant of transport cross section ****
double precision function intqt(q)

!variables
  double precision, intent(in):: q
  double precision potential
  double precision s

!calculation
  s=1.0d0
  if (allocated(coeffs)) then
     if (q<=coeffs(1,1)) s=coeffs(1,2)
     if (q>=coeffs(1,1).and. q<=coeffs(nsk,1) ) s=spl(q,coeffs,nsk)    
  end if

  potential=pot(q)/(4.0d0*pi*equadratat)
  intqt=q**3*s*potential**2
end function


!****** integrant of D_{ei} for DB *****
double precision function dei(k)
!only usable for Debye potential

implicit none
!variables
  double precision, intent(in)::k
  double precision qt
  double precision b, phi
  double precision vorf, f
  double precision hilf

!calculation of transport cross sections 
  if (typ_ei=='t' .and. k<=coeff(ntrq,1)) then
     if (k<coeff(1,1)) then 
        qt=coeff(1,2)
     else
        qt=spl(k, coeff, ntrq)
     end if
  else
     if (typ/=3) then                
        if (allocated(coeffs) .and. typ==2) then
           lb=0.0d0
           err=-1
           phi=d01ahf(lb, 2*k, 1.0d-6, neval, esterr, coulog,5000,err)
!That's not functioning because of coulog=coulog(x,y)!!!
        else                            
           phi=0.0d0
           b=4.0d0*k**2/kappa**2
           phi=0.5d0*(dlog(b+1.0d0)-b/(b+1.0d0))
        end if
        qt=4.0d0*pi*charge/k**4*phi
     end if
  end if
  if (typ==3) qt=qtrr(k, rd)

!determination of prefactor and f(1-f)
  vorf=dd/(ne*thermlambda**3*pi)
  f=0.0d0
  if (k<dsqrt((5.0d1+muekt)*kbt)) then
     hilf=dexp(k**2/kbt-muekt)
     hilf=1.0d0/hilf
     f=hilf/(1.0d0+hilf)**2
  end if

!calculation of dei
  dei=vorf*k*k**2*(k**2/kbt)**(dim+1)*f*qt

!*************
contains
 
!Fit Roepke/Redmer
double precision function qtrr(k, rd)       

!variables
  double precision, intent(in):: k,rd
  double precision hilf1, hilf2, qt1, qt2, qt3
  double precision, parameter:: a1=0.5389d0, a2=1.9804d0, a3=3.4035d0, &
       a4=2.0298d0, a5=0.6276d0, a6=4.5d0

!calculation
  hilf1=k**2*rd/2
  hilf2=(k/pi)**2
  qt1=1.0d0+a5*dlog(a6/hilf1+1.0d0)
  qt2=(1+a4*hilf2)/(a1+a2*hilf2+a3*hilf2**2)
  qt3=qt2*hilf1**2*qt1**2
  qtrr=2*pi/k**4*dlog(qt3+1.0d0)

end function qtrr

end function dei

!****** Coulomb logarithm **************
double precision function coulog(k)      

!variables
  double precision, intent(in):: k
  double precision eps, v, str

!calculation
  eps=1.0d0+1.0d0/(rd*k)**2
  v=1/k**2
  str=1.0d0
  if (k<=coeffs(1,1)) str=coeffs(1,2)
  if (k>=coeffs(1,1).and. k<=coeffs(nsk,1) ) str=spl(k,coeffs,nsk)    
  coulog=k**3*str*v**2/eps**2
  
end function coulog
 

!********* integrant of D_{ee} **********

double precision function dee(k)

implicit none
!variables
  double precision, intent(in)::k
  double precision qt
  double precision vorf, f
  double precision p, b
  integer i,j

!initializing
  p=2.0d0*k**2/kbt

!calculation of transport cross sections
  if (typ_ee=='t' .and. k<=coeffe(ntrqe,1)) then
     if (k<coeffe(1,1)) then 
        qt=coeffe(1,2)
     else
        qt=spl(k, coeffe, ntrqe)
     end if
  else
     b=4.0d0*k**2/kappa**2
     qt=4.0d0*pi/k**4*((1.0d0+3.0d0/b-1.0d0/b/(b+2.0d0))*dlog(b+1.0d0)-2.5d0)
  end if

!determination of prefactor and factor from distribution function
  vorf=1.41421356/pi*dd
  if(dim1==1 .and. dim2==1) &
       &f=1.0d0
  if(dim1==1 .and. dim2==2) &
       &f=3.5d0+p
  if(dim1==2 .and. dim2==2) &
       &f=77.0d0/4.0d0+7.0d0*p+p**2
  if(dim1==1 .and. dim2==3) &
       &f=189.0d0/16.0d0+27.0d0/4.0d0*p+3.0d0/4.0d0*p**2
  if(dim1==2 .and. dim2==3) &
       &f=2835.0d0/32.0d0+783.0d0/16.0d0*p+75.0d0/8.0d0*p**2+3.0d0/4.0d0*p**3
  if(dim1==3 .and. dim2==3) &
       &f=9.0d0*(14553.0d0/256.0d0+1215.0d0/32.0d0*p+313.0d0/32.0d0*p**2&
       &+9.0d0/8.0d0*p**3+p**4/16.0d0)
  if(dim1==1 .and. dim2==4) &
       &f=48.0d0*(231.0d0/256.0d0+99.0d0/128.0d0*p+11.0d0/64.0d0*p**2&
       &+p**3/96.0d0)
  if(dim1==2 .and. dim2==4) &
       &f=96.0d0*(4389.0d0/1024.0d0+429.0d0/128.0d0*p+55.0d0/64.0d0*p**2&
       &+5.0d0/48.0d0*p**3+p**4/192.0d0)
  if(dim1==3 .and. dim2==4) &
       &f=288.0d0*(81081.0d0/8192.0d0+34155.0d0/4096.0d0*p+2717.0d0/1024.0d0&
       &*p**2+211.0d0/512.0d0*p**3+17.0d0/512.0d0*p**4+p**5/768.0d0)
  if(dim1==4 .and. dim2==4) &
       &f=1152.0d0*(525525.0d0/32768.0d0+127413.0d0/8192.0d0*p&
       &+48323.d0/8192.0d0*p**2+3465.0d0/3072.0d0*p**3+247.0d0/2048.0d0*p**4&
       &+11.0d0/1536.0d0*p**5+p**6/4608.0d0)

!calculation of dee
  dee=vorf*k*k**2*p**2*dexp(-p)*f*qt

end function dee


!********** integrant of D_{ea} **************

double precision function dea(k)

  implicit none

!variables
  double precision, intent(in)::k
  double precision qt
  double precision vorf, f
  double precision hilf, hilfpol, pn
  double precision setA,setB,setC,setD,setE
  double precision kr0,kr0squ,kr0cub,kr0fou
  integer, dimension(dim+1)::vektor
  integer i

!case of no neutral particles
  if (nat+nmo==0.0d0) then
     dea=0.0d0
     return
  end if

!scattering cross sections qt
  if (typ/=3) then                
     if (typ_ea=='t' .and. k<=coeffa(ntrqa,1)) then 
        if (k<coeffa(1,1)) then 
           qt=coeffa(1,2)
        else
           qt=spl(k, coeffa, ntrqa)
        end if
     else
        hilfpol=(pi*pol/(8.0d0*rpol))**2
        if (typbn==1 .or. typ_ea=='t')then

           kr0=rpol/rdea
           kr0squ=kr0*kr0
           kr0cub=kr0*kr0*kr0
           kr0fou=kr0*kr0*kr0*kr0

           setA=1.0+2.0*kr0+7.0/pi**2*kr0squ+pi/7.0*kr0cub
           setB=exp(-18.0*kr0)
           setC=(1.0+22.0*kr0-11.3*kr0squ+33.0*kr0fou)&
               /(1.0+6.0*kr0+4.7*kr0squ+2.0*kr0fou)
           setD=(1.0+28.0*kr0+13.8*kr0squ+3.2*kr0fou)&
               /(1.0+8.0*kr0+10.0*kr0squ+kr0fou)
           setE=1.0+0.1*kr0+0.3665*kr0squ

           qt=setA*setA+3.0*setB*k*rpol+7.5*setC*k*k*rpol*rpol&
                -3.4*setD*(k*rpol)**3+10.6668*setE*(k*rpol)**4
           qt=pi**3*pol**2/4.0/rpol**2/qt
        end if
        if (typbn==2) then
           hilf=1.0d0/4.0d0/kbt
           do i=0,dim
              vektor(i+1)=(hilf*k**2)**(i)/faculty(i)
           end do
           qt=sum(vektor)
           qt=qt*faculty(dim)
        end if
     end if
  end if
  if (typ==3) qt=qtmb(k, rdea)

!determination of prefactor and f(1-f)
  if (typ_ea=='b' .and. typbn==2) then
     vorf=hilfpol*dd/(planckquerat*k**2/kbt)**(dim+1)
     f=dexp(-(0.25d0*k**2/kbt+2.0d0*rpol*k))/(1.0d0+1.0d0/(rdea**2*k**2))**2
  else
     vorf=(ne*thermlambda**3*pi)**(-1)*dd
     f=(1.0d0+dexp(-(k**2/kbt-muekt)))**(-2)*dexp(-(k**2/kbt-muekt))
  end if

!calculation of dea
  dea=vorf*k*k**2*(planckquerat*k**2/kbt)**(dim+1)*f*qt
  if(tabmo) then 
     dea=dea*(nat+nmo)/ne
  else
     dea=dea*nat/ne
  end if

!*********
contains

!Meyer-Bartoli-Fit
double precision function qtmb(k,rd)            

!variables
  double precision, intent(in):: k, rd
  double precision qt1, qt2, qt3
  double precision, parameter:: a1=80.6d0, a2=23.7d0, a3=42.3d0, a4=133.0d0  

!calculation
  qt1=(1.0d0+k**2/2.0d0-k**4.0d0/6.0d0)/(1.00+k**2)**3.0d0
  qt2=dlog(1.0d0+k**2)/k**2-qt1
  qt3=(1.0d0+dexp(-50.0d0*k**2))*(1.0d0+a1*k**2+a2*k**4)&
       /(1.0d0+a3*k**2+a4*k**4)
  qtmb=35.2d0/k**2*qt3*qt2

end function qtmb

end function dea


!********* generalized particle numbers *******
double precision function nn(n)

implicit none

!vriables
  integer, intent(in)::n
  double precision, parameter:: wpi=1.772453851d0
  double precision fak
  real n1

!calculation
  n1=(n+5.0d0)/2.0d0
  if (int(n1)==n1) fak=faculty(int(n1)-1)
  if (int(n1)/=n1) fak=halffaculty(n1-1)

  nn=8.0d0/3.0d0/wpi/thermlambda**3*fak*ferm(muekt, (n+1.0)/2.0)

end function nn


!************************ transport coefficients ***************************

subroutine calculation(d,res)
! sigma,lambda,kappa first in atomic units, then in SI

implicit none
!in- and output
  double precision, dimension(:,:), intent(in):: d
  double precision, dimension(:), intent(out):: res

!matrices and elements
  double precision, dimension(:,:),allocatable::det_s,det_k,det_l
  double precision enthalpie, n0

!parameter for change AU->SI
  double precision, parameter::lsi=4.59984757d6, psi=-8.6173d-2, wsi=3.41577d-2
  !(lsi contains also e^2; psi gives mV/K for thermopower)

!helping variables
  double precision dt0, dts, dtk, dtl
  integer i,col,rang,j
  double precision, dimension(:,:), allocatable::help_td

!******** start ****************
  !matrices: initilizing & generalized particle numbers
  rang=size(d(1,:))
  n0=nn(0)
  enthalpie=nn(2)/n0

  allocate(help_td(2:rang+1,2))
  do i=2,rang+1
     col=2*(i-1)
     help_td(i,1)=nn((2*(i-2)))/n0
     help_td(i,2)=(nn(col)-enthalpie*nn((2*(i-2))))/n0
  end do

  !matrices: determination
  allocate(det_s(rang+1,rang+1))
  det_s(1,1)=0.0d0
  det_s(2:rang+1,1)=help_td(2:rang+1,1)
  det_s(1,2:rang+1)=det_s(2:rang+1,1)
  det_s(2:rang+1,2:rang+1)=d
  if (do_trans(2) .or. do_trans(3)) then
     allocate(det_k(rang+1,rang+1))
     det_k(1,1)=0.0d0
     det_k(2:rang+1,1)=help_td(2:rang+1,1)
     det_k(1,2:rang+1)=help_td(2:rang+1,2)
     det_k(2:rang+1,2:rang+1)=d
  end if
  if (do_trans(2))then
     allocate(det_l(rang+1,rang+1))
     det_l(1,1)=0.0d0
     det_l(2:rang+1,1)=help_td(2:rang+1,2)
     det_l(1,2:rang+1)=det_l(2:rang+1,1)
     det_l(2:rang+1,2:rang+1)=d
  end if
  deallocate(help_td)

  !matrices: determinants
  dt0=det(d)
  dts=det(det_s)
  if (allocated(det_l)) dtl=det(det_l)
  if (allocated(det_k)) dtk=det(det_k)

  i=1
  if (do_trans(1)) then
     res(i)=-dts/dt0*lsi
     i=i+1
  end if
  if (do_trans(2)) then
     res(i)=-(dtl-dtk**2/dts)/dt0*wsi*temp
     i=i+1
  end if
  if (do_trans(3)) then
     res(i)=dtk/dts*psi
     i=i+1
  end if
  if (do_trans(4)) res(i)=1.3467d8*res(2)/temp/res(1)
end subroutine calculation

!************* reflectivity ******************
double precision function refl(sigma_0,omega)
!Reflectivity from Frenel formula with simple Drude model. Input in au-units!

implicit none
!variables
  double precision, intent(in):: sigma_0,omega
  double precision:: omega_plsq, frac_omsq
  complex:: sqr_eps, chi, comprefl
  complex:: imath=(0.0,1.0)
  
!calculation
  omega_plsq=16.0*pi*ne
  frac_omsq=omega*omega/omega_plsq

  chi=imath*omega/4.0/pi/sigma_0+frac_omsq
  chi=-1.0/chi
  
  sqr_eps=1.0+chi
  sqr_eps=csqrt(sqr_eps)

  comprefl=(sqr_eps-(1.0,0.0))/(sqr_eps+(1.0,0.0))
  refl=cabs(comprefl)**2

end function refl


end module transmod

