!******************************************************************************
!****************************** PROGRAMM TRANS_LRT ****************************
!******************************************************************************

!******************************************************************************
!COMPILER: 
! f90 trans_LRT.f90 trqa.f90 phasen.c -lmathlib -lkernlib -lnag -o trans_LRT.x
!
!This program calculates the transport coefficients electrical conductivity, 
!thermal conductivity, thermopower, and reflectivity of plasmas within Linear 
!Response Theory in the formulation of Zubarev (LRT). For details see
! D. N. Zubarev, "Nonequilibrium Statistical Thermodynamics", Plenum Press
!   New York 1974; D. N. Zubarev et al., "Statistical Mechanics of 
!   Nonequlibrium Processes", Akademie Verlag, Berlin 1996
! F. E. Hoehne et al., Physica A 128 (1984) 643
! H. Reinholz et al., PRE 52 (1995) 5368
! R. Redmer, PRE 59 (1999) 1073; S. Kuhlbrodt, PRE 62 (2000) 7191; 
! S. Kuhlbrodt, Doktoral Thesis, Rostock 2003 
!
!Input files are "elementfile", "intransfiles", and "compfile", containing data
!about polarization, parameters for the calculation and the plasma composition.
!For example, the output file of comp_PIP.f90 can serve as such a composition 
!file. As output, three files can be given: Default output is "outtransfile", 
!giving electrical and thermal conductivity, thermopower, and Lorentz number. 
!In "outtrans_sesa", all of these coefficients except Lorentz number are given 
!in sesame format. "outtrans_refl" will provide data for reflectivities.
!
! S. Kuhlbrodt 2003
!******************************************************************************

!used modules
include "mod_constants.f90"
include "mod_basics.f90"
include "mod_math.f90"
include "mod_trans.f90"
include "mod_fermis.f90"

!******************************************************************************
!******************************************************************************
program trans_LRT
 
use basics
use transmod

!************************** VARIABLES AND PARAMETER ***************************
implicit none
!file parameter
  character(len=10), parameter:: trqdir='trqdirlnk/'
  character(len=10), parameter:: &
       potdir='potdirlnk/'
  character(len=11), parameter:: elementfile='elementfile'
  character(len=11), parameter:: infile='intransfile'
  character(len=8), parameter:: compfile='compfile'
  character(len=12), parameter:: outfile='outtransfile'
  character(len=13), parameter:: out_sesa='outtrans_sesa'
  character(len=13), parameter:: out_refl='outtrans_refl'
  character(len=100) fstr

!material paramter (some are also definied in mod_trans.f90)
  integer ordz, anzele
  double precision atmass,metomi
  
!parameter for density/temperature loop and grid
  double precision, dimension(:,:,:),allocatable,target:: alpha_grid
  double precision, dimension(:,:),allocatable,target:: dens_grid
  double precision, dimension(:),allocatable,target:: temp_grid
  double precision, dimension(:,:),allocatable,target:: mol_grid
  double precision, dimension(:,:),allocatable,target:: trans_grid
  double precision, dimension(:,:,:),allocatable,target:: result_grid
  double precision, dimension(:),allocatable:: alpha
  double precision, pointer:: rho,n_heavy,iograd,beta_mo
  integer:: n_dens, n_temp, n_count, t_count,minion
  integer,dimension(:),allocatable,target:: min_Z, max_Z
  integer,pointer:: Z_min, Z_max

!for reflectivities
  double precision, dimension(:),allocatable,target:: omega_grid,wavel_grid
  double precision, dimension(:,:,:),allocatable:: refl_grid
  double precision, pointer:: omega,wavel,sigma
  integer n_wavel,w_count
  logical reflect

!instability parameter
  double precision, dimension(:,:,:),allocatable,target::dens_instab
  double precision, dimension(:,:,:),allocatable,target::alpha_instab
  double precision, dimension(:,:,:),allocatable,target::result_instab
  double precision, dimension(:,:),allocatable,target::mol_instab
  integer, dimension(:),allocatable:: pos_instab, len_instab

!calculation parameters
  integer, parameter:: ordn=2                         
  logical tabt, tabe, taba, tabref, molecules
  integer n_coeff,maxion
  logical control

!variables for transport cross sections
  character(len=13), dimension(:), allocatable::feldqd
  character(len=13):: feldqe
  character(len=20):: feldqa
  character(len=18) potfile
  double precision, dimension(:), allocatable:: ktrq,trq,ktrqe,trqe,ktrqa,trqa
  double precision, dimension(:), allocatable:: qstr,str
  double precision, dimension(:), allocatable:: qpot,epot

!variables for correlation functions (some are also in mod_trans.f90)
  double precision, dimension(ordn+1,ordn+1):: korrei, korree, korrea
  double precision, dimension(2*ordn+1):: korrhilf, korrhilf2
  double precision, dimension(ordn+1, ordn+1):: d
  integer u, col

!variables for output
  logical default_output
  character(len=11), dimension(4):: out_str,unit_str

!helping variables
   double precision::help, ionpart, kappaquad, hilfrd
   double precision, dimension(:), allocatable::help_od
   double precision, dimension(:,:,:), allocatable::coeffmatrix, hilfcoeff
   integer i,j, hilftyp, line, column, m, n_ende, schritt, lauf,shift
   character(len=30) text
   character(len=1) text1,text2,text3,text4
   logical hilftab

!external routines
   double precision dGauss
   external mktrq

!  double precision  sigma, lambda, thermpow, reflec, lz


!*********************************** START ************************************
control=.false.
!control=.true.


!********************** INPUT AND FIRST INITIALIZATIONS ***********************


!********* composition file I ********
  if (control) print*, 'reading from compfile I'

  open (10, file=compfile, status='old', access='sequential')

  !kind of output
  read(10,*) text, text
  if (text=='y') then 
     default_output=.true.
  else
     default_output=.false.
  end if

  !material
  read(10,*) text, ordz

  !with/without molecules
  read(10,*) text, text
  if (text=='y') then
     molecules=.true.
  else
     molecules=.false.
  end if

!**************** elementfile ****************
  if (control) print*, 'reading elementfile'

  open(11,file=elementfile, status='old', access='sequential')
  allocate(help_od(1:100))
  read(11,*) text,anzele
  i=0
  line=1
  do while (i/=ordz .and. line<=anzele)
     read(11,*) text,i,atmass,metomi,pol,maxion,help_od(1:maxion)
     line=line+1
  end do
  close(11, status='keep')
  deallocate(help_od)

  if (line>anzele) then
     write(*,900) ordz
     stop
  else
     !radius for polarization potential
     rpol=(pol/ordz**(1.0d0/3.0d0)/2.0d0)**0.25d0      ! Mittelman/Watson
     if (ordz==1) rpol=1.4565d0
  end if

900 format&
   &('Sorry, parameter for element ',I2,' could not be found in elementfile.'/&
   & 'Please check ordering number in compfile or modify elementfile!')

!********** composition file 2 **************
  if (control) print*, 'reading from compfile II'

  !density/temperature grid parameter
  read(10,*) text,text,n_temp
  read(10,*) text,text,n_dens
  allocate(temp_grid(1:n_temp),dens_grid(1:n_dens,2))
  allocate(alpha_grid(n_dens,-1:maxion,n_temp))
  temp_grid=0
  dens_grid=0
  alpha_grid=0
  if (molecules) then
     allocate(mol_grid(n_dens,n_temp))
     mol_grid=0
  end if

  !reading of alphas
  if (default_output) then
     allocate(alpha_instab(50,-1:maxion,n_temp),dens_instab(50,2,n_temp))
     allocate(pos_instab(1:n_temp),len_instab(1:n_temp))
     allocate(min_Z(1:n_temp),max_Z(1:n_temp))
     alpha_instab=0
     dens_instab=0
     pos_instab=0
     len_instab=0
     if (molecules) then
        allocate(mol_instab(50,n_temp))
        mol_instab=0
     end if
     do t_count=1,n_temp
        read(10,*) text
        read(10,*) text, text, temp_grid(t_count)
        read(10,*) text, pos_instab(t_count), len_instab(t_count)
        read(10,*) text, min_Z(t_count), text,max_Z(t_count) 
        read(10,*) text
        read(10,*) text
        Z_min=>min_Z(t_count)
        Z_max=>max_Z(t_count)

        do n_count=1,pos_instab(t_count)
           if (molecules) then
              read(10,*) dens_grid(n_count,:),alpha_grid(n_count,-1,t_count),&
                   alpha_grid(n_count,Z_min:Z_max,t_count),&
                   mol_grid(n_count,t_count)
           else
              read(10,*) dens_grid(n_count,:),alpha_grid(n_count,-1,t_count),&
                   alpha_grid(n_count,Z_min:Z_max,t_count)
           end if
        end do

        if (len_instab(t_count)/=0) then
           do n_count=1,len_instab(t_count)
              if (molecules) then
                 read(10,*) dens_instab(n_count,:,t_count),&
                      alpha_instab(n_count,-1,t_count),&
                      alpha_instab(n_count,Z_min:Z_max,t_count),&
                      mol_instab(n_count,t_count)
              else
                 read(10,*) dens_instab(n_count,:,t_count),&
                      alpha_instab(n_count,-1,t_count),&
                      alpha_instab(n_count,Z_min:Z_max,t_count)
              end if
           end do
           do n_count=pos_instab(t_count)+1,n_dens
              if (molecules) then
                 read(10,*) dens_grid(n_count,:),&
                      alpha_grid(n_count,-1,t_count),&
                      alpha_grid(n_count,Z_min:Z_max,t_count),&
                      mol_grid(n_count,t_count)
              else
                 read(10,*) dens_grid(n_count,:),&
                      alpha_grid(n_count,-1,t_count),&
                      alpha_grid(n_count,Z_min:Z_max,t_count)
              end if
           end do
        end if
        nullify(Z_min,Z_max)
     end do
  else
     allocate(min_Z(1:n_dens),max_Z(1:n_dens))
     do n_count=1,n_dens
        read(10,*) text
        read(10,*) text, text, dens_grid(n_count,1)
        read(10,*) text, min_Z(n_count),text,max_Z(n_count) 
        read(10,*) text
        read(10,*) text
        Z_min=>min_Z(n_count)
        Z_max=>max_Z(n_count)
        do t_count=1,n_temp
           if (molecules) then
              read(10,*) temp_grid(t_count), alpha_grid(n_count,-1,t_count),&
                   alpha_grid(n_count,Z_min:Z_max,t_count),&
                   mol_grid(n_count,t_count)
           else
              read(10,*) temp_grid(t_count), alpha_grid(n_count,-1,t_count),&
                   alpha_grid(n_count,Z_min:Z_max,t_count)
           end if
        end do
        nullify(Z_min,Z_max)
    end do
    dens_grid(:,2)=dens_grid(:,1)*1.0d-3/atmass/atommasse
  end if
  close(10,status='keep')

!********* reading of intransfile *************
  if (control) print*, 'reading of intransfile'

  open (10, file=infile, status='old', access='sequential')
  read(10,*) text
  read(10,*) text

  !which transport coefficients?
  read(10,*) text1,text2,text3,text4
  do_trans=.false.
  if (text1=='y') then
     do_trans(1)=.true.
  end if
  if (text2=='y') then
     do_trans(2)=.true.
  end if
  if (text3=='y') then
     do_trans(3)=.true.
  end if
  if (text4=='y') then
     if (do_trans(1) .and. do_trans(2)) then
        do_trans(4)=.true.
     else
        print*, 'Warning: Lorentz number cannot be calculated, if sigma or'
        print*, 'lambda is missing.'
     end if
  end if
  i=0
  if (do_trans(1)) then
     i=i+1
     out_str(i) ='   sigma   '
     unit_str(i)=' 1/(ohm*m) '
  end if
  if (do_trans(2)) then
     i=i+1
     out_str(i) ='  lambda   '
     unit_str(i)='  W/(Km)   '
  end if
  if (do_trans(3)) then
     i=i+1
     out_str(i) ='   kappa   '
     unit_str(i)='   mV/K    '
  end if
  if (do_trans(4)) then
     i=i+1
     out_str(i) =' lorentz   '
     unit_str(i)='           '
  end if
  n_coeff=i

  !pottype of e-i scattering / initializations of needed parameters
  screened_pot=.false.
  read(10,*) pottype
  select case (pottype)
     case('DB')
        read(10,*)
        screened_pot=.true.
     case('SC')
        read(10,*)
     case('AS')
        read(10,*) radius
     case('PP')
        read(10,*) potfile,text
        if (text=='y') screened_pot=.true.

        !reading of pseudopotential and spline interpolation
        open(11,file=potdir//potfile, status='old', access='sequential')
        read(11,*) text, npot
        allocate(qpot(npot),epot(npot),coeffpot(npot,5))
        qpot=0.0d0
        epot=0.0d0
        coeffpot=0.0d0

        do i=1,npot
           read(11,*) qpot(i),epot(i)
        end do
        close(11, status='keep')

        call cubspl(qpot, epot, npot, coeffpot)
        deallocate(qpot, epot)
     end select

  !type of scattering
  tabt=.false.
  tabe=.true.
  taba=.true.
  tabmo=.false.
  typ=2
  read(10,*) typ_ei,typ_ee,typ_ea,typ_em
  select case(typ_ei)
  case('t')
     if (pottype=='DB') then
        tabt=.true.
        typ=1
     else
     print*, 'T-matrix only available for e-i scattering with Debye potential.'
        print*, 'Type of scattering changed to Born approximation.'
        typ_ei='b'
     end if
  case('n')
     print*, 'e-i scattering can not be neglected. Please prove intransfile!'
     stop
  end select
  select case(typ_ee)
  case('b')
     if (tabt) then
    print*, 'e-e scattering must be treated in the same way as e-i. (T-matrix)'
        typ_ee='t'
     end if
  case('n')
     tabe=.false.
  end select
  select case(typ_ea)
  case('t')
     print*, 'Warning: T-matrix was not tested for e-a scattering'
  case('b')
     typbn=1
  case('n')
     taba=.false.
  end select
  if (molecules) then
     select case(typ_em)
     case('t')
        print*, 'T-matrix cannot be used for e-m scattering!'
        print*, 'Approximation type changed to Born approximation.'
        typ_em='b'
        tabmo=.true.
     case('b')
        tabmo=.true.
     end select
     if (tabmo .and. ordz/=1) then
        print*, 'Warning: Polarization of atom and molecule are considered as'
        print*, 'same value!'
     end if
  end if

  !reflectivities
  read(10,*) n_wavel
  reflect=.false.
  if (n_wavel/=0) reflect=.true.
  if (reflect) then
     if (.not. do_trans(1)) then
        print*, 'The calculation of reflectivities needs electrical'
        print*, 'conductivities. sigma will be calculated.'
        do_trans(1)=.true.
     end if
     allocate(wavel_grid(1:n_wavel), omega_grid(1:n_wavel))
     allocate(refl_grid(n_dens,n_wavel,n_temp))
     do w_count=1,n_wavel
        read(10,*) wavel_grid(w_count)
     end do
     omega_grid=2.0d0*pi*2.99792458d8/wavel_grid*1.0d9
     omega_grid=omega_grid*zeitat
  end if

  if (control) print*, 'ready with input'

!**************************** START OF CALCULATIONS **************************
  if (control) print*, ''
  if (control) print*, 'start of calculations'

  allocate(result_grid(n_dens,n_coeff,n_temp))
  if (default_output) then
     if (maxval(len_instab)/=0) &
       allocate(result_instab(maxval(len_instab),n_coeff,n_temp))
  end if

!************ temperature loop ****************
  if (control) print*, 'start of temperature loop'

  do t_count=1,n_temp

     !first initializations
     temp=>temp_grid(t_count)
     kbt=kbat*temp
     thermlambda=2.0d0*pi*planckquerat/(dsqrt(2.0d0*pi*kbt*emasseat))

     !***************** density loop *****************
     if (control) print*, 'start of density loop'
     
     n_ende=n_dens
     if (default_output) n_ende=n_ende+len_instab(t_count)

     do n_count=1,n_ende
        
     !initializing
        if (control) print*, 'initializing of composition'

        !Z_min/Z_max
        if(default_output) then
           Z_min=>min_Z(t_count)
           Z_max=>max_Z(t_count)
        else
           Z_min=>min_Z(n_count)
           Z_max=>max_Z(n_count)
        end if
        minion=max(Z_min,1)

        allocate(alpha(Z_min:Z_max))
        if (n_count<=n_dens) then
           alpha = alpha_grid(n_count,Z_min:Z_max,t_count)
           iograd => alpha_grid(n_count,-1,t_count)
           n_heavy => dens_grid(n_count,2)
           if(molecules) beta_mo => mol_grid(n_count,t_count)
        else
           alpha = alpha_instab(n_count-n_dens,Z_min:Z_max,t_count)
           iograd => alpha_instab(n_count-n_dens,-1,t_count)
           n_heavy => dens_instab(n_count-n_dens,2,t_count)
           if(molecules) beta_mo => mol_instab(n_count-n_dens,t_count)
        end if

        ne=iograd*n_heavy*abohr**3*1.0d6
        nat=alpha(0)*n_heavy*abohr**3*1.0d6
        if(molecules) nmo=beta_mo*n_heavy*abohr**3*1.0d6
        help=ne*thermlambda**3/2.0d0

        kappaquad=equadratat/epsilon0at/kbt&
             &*ne/help*fermdach(help, -0.5)
        kappa=dsqrt(kappaquad)
        rd=1.0d0/kappa
        rdei=rd
        if (tabe) rdee=rd
        if (taba) rdea=rd

        call transpar

        !transport cross sections in T-matrix
        if (typ_ei=='t') then
           if (control) print*, 'e-i transport cross sections in TM'

           !allocation of trq-files
           allocate(feldqd(minion:Z_max),ntrq_arr(minion:Z_max))
           ntrq_arr=0

           !creation and reading of trq-files: e-i scattering
           if (control) print*, '  - creation and reading of trq-files'
           hilfrd=min(5.0d2,rd) 
           do j=minion,Z_max
              if (alpha(j)>1.0d-6 .or. Z_max==1) then
                 call mktrq(hilfrd,dble(j),0.0d0,0.0d0,'DB',trqdir,feldqd(j))
                 fstr=trqdir//'DB/'//feldqd(j)
                 open(15,file=fstr,status='old',access='sequential')
                 read(15,*) text
                 read(15,*) text
                 read (15, *) text, ntrq_arr(j)
                 close(15, status='keep')
              end if
           end do
           !spline interpolation
           if (control) print*, '  - spline interpolation'
           allocate(coeffmatrix(maxval(ntrq_arr),1:5,minion:Z_max))
           do j=minion,Z_max
              if (alpha(j)>1.0d-6 .or. Z_max==1) then
                 ntrq=ntrq_arr(j)
                 allocate(ktrq(ntrq), trq(ntrq), coeff(ntrq,5))
                 fstr=trqdir//'DB/'//feldqd(j)
                 open(15,file=fstr,status='old',access='sequential')
                 do lauf=1,3
                    read(15,*) text
                 end do
                 do lauf=1,ntrq
                    read(15,*) ktrq(lauf), trq(lauf)
                 end do
                 close(15, status='keep')
                 call cubspl(ktrq, trq, ntrq, coeff)
                 coeffmatrix(1:ntrq,1:5,j)=coeff(1:ntrq,1:5)
                 deallocate(ktrq, trq, coeff)
                 hilfrd=rd
              end if
           end do
           if (control) print*, 'qdb-file(s) + coeffmatrix created/read'
        end if

        if (typ_ee=='t') then
           if (control) print*, 'e-e transport cross sections in TM'
           hilfrd=min(5.0d2,rd) !
           !creation and reading of trq-files: e-e scattering
           call mktrq(hilfrd,1.0d0,0.0d0,0.0d0,'EE',trqdir,feldqe)
           fstr=trqdir//'EE/'//feldqe
           open(15,file=fstr,status='old',access='sequential')
           read(15,*) text
           read(15,*) text
           read (15, *) text, ntrqe
           allocate(ktrqe(ntrqe), trqe(ntrqe))
           ktrqe=0.0d0
           trqe=0.0d0
           do lauf=1,ntrqe
              read(15,*) ktrqe(lauf), trqe(lauf)
           end do
           close(15, status='keep')
           !spline interpolation
           allocate(coeffe(ntrqe,5))
           coeffe=0.0d0
           call cubspl(ktrqe, trqe, ntrqe, coeffe)
           deallocate(ktrqe, trqe)
           if (control) print*, 'qee-file + coeffe created/read'
        hilfrd=rd
        end if

        if (typ_ea=='t' .and. ordz/=1) then    !(hydrogen with Meyer-Bartholy))
           if (control) print*, 'e-a transport cross sections in TM'
           !creation and reading of trq-files: e-a scattering
           call mktrq(hilfrd,0.0d0,pol,rpol,'PP',trqdir,feldqa)
           fstr=trqdir//'PP/'//feldqa
           open(15,file=fstr,status='old',access='sequential')
           read(15,*) text
           read(15,*) text
           read (15, *) text, ntrqa
           allocate(ktrqa(ntrqa), trqa(ntrqa))
           ktrqa=0.0d0
           trqa=0.0d0
           do lauf=1,ntrqa
              read(15,*) ktrqa(lauf), trqa(lauf)
           end do
           close(15, status='keep')
           !spline interpolation
           allocate(coeffa(ntrqa,5))
           coeffa=0.0d0
           call cubspl(ktrqa, trqa, ntrqa, coeffa)
           deallocate(ktrqa, trqa)
           if (control) print*, 'qea-file + coeffa created/read'
        hilfrd=rd
        end if
     
        !correlation functions
        if (control) print*, 'correlation functions'
        !e-i correlation
        korrei=0.0d0
        korrhilf2=0.0d0
        hilftyp=0
        do j=minion,Z_max
           if (alpha(j)>1.0d-6 .or. Z_max==1) then
              charge=dble(j)
              if (typ_ei=='t') then
                 ntrq=ntrq_arr(j)
                 allocate(coeff(ntrq,5))
                 coeff=0.0d0
                 coeff(1:ntrq,1:5)=coeffmatrix(1:ntrq,1:5,j)
              end if
              korrhilf=0.0d0
              do dim=0,2*ordn
                 lb=1.0d-3
                 ub=1.0d1 
                 err=-1
                 if (pottype=='DB') then 
                    korrhilf(dim+1)&
                         =d01ahf(lb,ub,1.0d-6,neval,esterr,dei,5000,err)
                    if (korrhilf(dim+1)/=0.0d0) then
                       call testgrenze('o',ub,1.0d-6,korrhilf(dim+1),dei)
                       call testgrenze('u',lb,1.0d-6,korrhilf(dim+1),dei)
                    end if
                 else
                    korrhilf(dim+1)=dGauss(psdei,lb,ub,1.0d-6)
                    if (korrhilf(dim+1)/=0.0d0) then
                       call testgrenze('o',ub,1.0d-6,korrhilf(dim+1),psdei)
                       call testgrenze('u',lb,1.0d-6,korrhilf(dim+1),psdei)
                    end if
                 end if
              end do
              korrhilf=alpha(j)/sum(alpha(minion:Z_max))*dsqrt(korrhilf)
              korrhilf2=korrhilf2+korrhilf
              if (allocated(coeff)) deallocate(coeff)
           end if
        end do
        korrhilf2=korrhilf2**2
        do dim=1,ordn+1
           korrei(dim,1:ordn+1)=korrhilf2(dim:ordn+dim)
        end do
        if (allocated(coeffmatrix)) deallocate(coeffmatrix)
        if (control) print*, 'ready with e-i correlation'

        !e-e correlation
        if (tabe) then
           if (control) print*, 'e-e correlation'
           korree=0.0d0
           hilftyp=0
           do dim1=1,ordn
              do dim2=dim1,ordn
                 lb=1.0d-3
                 ub=1.0d1
                 err=-1
                 korree(dim1+1,dim2+1)=&
                      &d01ahf(lb,ub,1.0d-6,neval,esterr,dee,5000,err)
                 if (korree(dim1+1,dim2+1)/=0.0d0) then
                    call testgrenze('o',ub,1.0d-6,korree(dim1+1,dim2+1),dee)
                    call testgrenze('u',lb,1.0d-6,korree(dim1+1,dim2+1),dee)
                 end if
                 korree(dim2+1,dim1+1)=korree(dim1+1,dim2+1)
              end do
           end do
           if (allocated(coeffe)) deallocate(coeffe)
           if (control) print*, 'ready with e-e correlation'
        end if

        !e-a/e-m correlation
        if (taba) then
           if (control) print*, 'e-a/e-m correlation'
           korrea=0.0d0
           korrhilf=0.0d0
           hilftyp=0
           if (typ_ea=='t' .and. ordz==1) then
              hilftyp=typ
              typ=3
              typbn=1
           end if

           do dim=0,2*ordn
              lb=1.0d-3
              ub=1.0d1 
              err=-1
              korrhilf(dim+1)=d01ahf(lb,ub,1.0d-6,neval,esterr,dea,5000,err)
              if (korrhilf(dim+1)/=0.0d0) then
                 call testgrenze('o',ub,1.0d-6,korrhilf(dim+1),dea)
                 call testgrenze('u',lb,1.0d-6,korrhilf(dim+1),dea)
              end if
           end do
           if (hilftyp==1) typ=hilftyp

           do dim=1,ordn+1
              korrea(dim,1:ordn+1)=korrhilf(dim:ordn+dim)
           end do
           if (allocated(coeffa)) deallocate(coeffa)
           
           if (control) print*, 'ready with e-a/e-m correlation'
        end if

        !calculations of transport coefficients
        if (control) print*, 'calculations of transport coefficients'
        dim=ordn+1

        !matrices: correlations
        d=korrei
        if (tabe) d=d+korree
        if (taba) d=d+korrea

        if (n_count<=n_dens) then
           call calculation(d,result_grid(n_count,:,t_count))
           if (reflect) then
              sigma=>result_grid(n_count,1,t_count)
              do i=1,n_wavel
                 omega=>omega_grid(i)
                 refl_grid(n_count,i,t_count)&
                      =refl(sigma/2.299924d6,omega)
                 nullify(omega)
              end do
           end if
        else
           call calculation(d,result_instab(n_count-n_dens,:,t_count))
        end if

        nullify(iograd,n_heavy,Z_min,Z_max)
        deallocate(alpha)
        if (tabt) deallocate(feldqd,ntrq_arr)

     end do
     if (control) print*, 'end of density loop'
     !********* end of density loop *************

  end do
  if (control) print*, 'end of temperature loop'
!********* end of temperature loop *************


!********************************* OUTPUT *************************************

!outfile
  if (control) print*, 'begin output with outfile'
  open(30,file=outfile, status='replace', access='sequential')

  !first lines
  if (default_output) then
     write(30,90) 'y'
  else
     write(30,90) 'n'
  end if
90 format('#default_form ',A2)
  write(30,100) ordz, n_temp, n_dens
100 format('#element ',I2,/&
         &'#n_T = ',I4,/&
         &'#n_rho = ',I4)
  
  if (default_output) then
     do t_count=1,n_temp
        write(30,101)
101     format('#-------------------------')
        write(30,110) int(temp_grid(t_count))
110     format('#T = ',I8,' K')
        write(30,120) pos_instab(t_count),len_instab(t_count)
120     format('#instability',2I4)

        write(30,130,advance="no")
        do i=1,n_coeff
           write(30,132,advance="no") out_str(i)
        end do
        write(30,*)
        write(30,131,advance="no")
        do i=1,n_coeff
           write(30,132,advance="no") unit_str(i)
        end do
        write(30,*)
        do n_count=1,pos_instab(t_count)
           write(30,140,advance="no") dens_grid(n_count,:),&
                alpha_grid(n_count,-1,t_count)
           do i=1,n_coeff
              write(30,141,advance="no") result_grid(n_count,i,t_count)
           end do
           write(30,*)
        end do
        if (len_instab(t_count)>0) then
           do n_count=1,len_instab(t_count)
              write(30,140,advance="no") dens_instab(n_count,:,t_count),&
                   alpha_instab(n_count,-1,t_count)
              do i=1,n_coeff
                 write(30,141,advance="no") result_instab(n_count,i,t_count)
              end do
              write(30,*)
           end do
           do n_count=pos_instab(t_count)+1,n_dens
              write(30,140,advance="no") dens_grid(n_count,:),&
                   alpha_grid(n_count,-1,t_count)
              do i=1,n_coeff
                 write(30,141,advance="no") result_grid(n_count,i,t_count)
              end do
              write(30,*)
           end do
        end if
     end do

130  format('#  rho      n_heavy    alpha_e')
131  format('# g/ccm      1/ccm            ')
132  format(A13)
140  format(E9.3,1x,E11.4,2x,F7.5)
141  format(E13.5)

  else
     do n_count=1,n_dens
        write(30,101)
        write(30,190) dens_grid(n_count,1)
190     format('#rho = ',E9.3,' g/ccm')
        
        write(30,160,advance="no")
        do i=1,n_coeff
           write(30,162,advance="no") out_str(i)
        end do
        write(30,*)
        write(30,161,advance="no")
        do i=1,n_coeff
           write(30,162,advance="no") unit_str(i)
        end do
        write(30,*)
        do t_count=1,n_temp
           write(30,170,advance="no") int(temp_grid(t_count)),&
                alpha_grid(n_count,-1,t_count)
           do i=1,n_coeff
              write(30,171,advance="no") result_grid(n_count,i,t_count)
           end do
           write(30,*)
        end do
     end do
     
160  format('#   T      alpha_e  ')
161  format('#   K               ')
162  format(A13)
170  format(I8,E11.3)
171  format(E13.5)
     
  end if
  close(30,status='keep')

!SESAME file
  if (control) print*, 'output to SESAME file'
  open(30,file=out_sesa, status='replace', access='sequential')
 
  write(30,300) ordz,atmass,real(n_dens),real(n_temp)
  write(30,310) dens_grid(:,1),temp_grid(:),&
       ((alpha_grid(n_count,-1,t_count),n_count=1,n_dens),t_count=1,n_temp),&
       (((result_grid(n_count,i,t_count),n_count=1,n_dens),&
       t_count=1,n_temp),i=1,n_coeff)
  close(30,status='keep')

300 format(I3,10x,3E13.5)
310 format(4E13.5)


!reflectivity file
  if (reflect) then
     open(30,file=out_refl, status='replace', access='sequential')
     if (control) print*, 'output to reflectivity file'
     
     if (default_output) then
        write(30,90) 'y'
     else
        write(30,90) 'n'
     end if
     write(30,100) ordz,n_temp,n_dens
     
     if (default_output) then
        do t_count=1,n_temp
           write(30,101)
           write(30,110) int(temp_grid(t_count))
           
           write(30,220,advance="no") 
           do i=1,n_wavel
              write(30,223,advance="no") wavel_grid(i)
           end do
           write(30,*)
           write(30,221,advance="no") 
           do i=1,n_wavel
              write(30,230,advance="no") 
           end do
           write(30,*)
           
           do n_count=1,n_dens
              write(30,222,advance="no") dens_grid(n_count,1)
              do i=1,n_wavel           
                 write(30,231,advance="no") refl_grid(n_count,i,t_count)
              end do
              write(30,*)
           end do
        end do
        
220     format('#    rho  ')
221     format('#   g/ccm ')
222     format(F9.3)
223     format(2x,F9.3,1x)
230     format('      nm    ')
231     format(E12.3)
        
     else
        do n_count=1,n_dens
           write(30,101)
           write(30,190) dens_grid(n_count,1)
           
           write(30,240,advance="no") 
           do i=1,n_wavel
              write(30,223,advance="no") wavel_grid(i)
           end do
           write(30,*)
           write(30,241,advance="no") 
           do i=1,n_wavel
              write(30,230,advance="no") 
           end do
           write(30,*)
           
           do t_count=1,n_temp
              write(30,242,advance="no") int(temp_grid(t_count))
              do i=1,n_wavel           
                 write(30,231,advance="no") refl_grid(n_count,i,t_count)
              end do
              write(30,*)
           end do
        end do
        
240     format('#    T   ')
241     format('#    K   ')
242     format(I8)
        
     end if
  close(30, status='keep')
end if

print*, 'ready with transport coefficients :-)'
end program
!***************************************************************************:-)
