!******************************************************************************
!This program determines transport cross sections in T-matrix approximation, 
!based on a program of S. Kosse (Greifswald). Scattering phase shifts are 
!calculated within Numerov-procedure, see modified C program of D. Gericke 
!(once Greifswald).
!
!For the calculation of phase shifts, the Schroedinger equation is made 
!dimensionless with  a_0=\hbar^2/(e^2 m_{ab}) (m_{ab}: reduced mass). Wave 
!numbers must be given in a_0. Transport cross sections are given originally 
!in a_0^2. In the outputfile, points (k(i),trq(i)) are given in 1/a_bohr and 
!a_bohr^2, respectively.
!
! S. Kuhlbrodt 2003
!*****************************************************************************

subroutine mktrq(rd,zeff,pol_ein,r0_ein,pot,zielvz,q)

implicit none 
!in- and output
  double precision, intent(in):: rd, zeff, pol_ein, r0_ein
  character(len=2), intent(in):: pot
  character(len=*), intent(in):: zielvz
  character(len=*), intent(out):: q

!potential/ file names
  character(len=3), parameter:: qdb='qdb', qee='qee', qpp='qpp'
  character(len=2) vstr
  character(len=len(q)) fstr1
  character(len=len(q)+len(zielvz)+3) fstr
  character(len=len(q)) hstr
  character(len=4) zstr,r0str
  character(len=6) polstr

  character(len=9), parameter:: eingangsfile='0-had.dat'
  character(len=9), dimension(8), parameter::&
       phasenfile=(/ '1-pha.dat','2-pha.dat','3-pha.dat','4-pha.dat',&
       '5-pha.dat','6-pha.dat','7-pha.dat','8-pha.dat'/)
  character(len=7), parameter:: k_l_file='k-l.dat'
  character(len=6), parameter:: in_file='in.dat'

!main variables
  integer:: l_max,anzk,anzl
  double precision z,pol,r0,kappa, length
  double precision kmin, kgrenz
  double precision, dimension(:), allocatable:: k_tm,qt_tm
  integer, dimension(:), allocatable::k_l_max
  double precision, dimension(:,:), allocatable::&
       phasen,phasen10,phasen100

!helping variables
  double precision, dimension(:,:), allocatable:: matrix
  double precision, dimension(:,:), allocatable:: delta, delta1
  double precision test
  real j_r,vorf
  integer i,j, column, hilf, hilfanz
  double precision, parameter::pi=3.141592654
  integer mag, lenhstr, lenfstr1
  integer abbruch, error
  character(len=40) text1
  character(len=15) text2,text3,text4,text5,text6
  character(len=1) enter
  external mainei


!********************************** Start ************************************

!****** name and open outputfile **********

  z=zeff
  pol=pol_ein
  r0=r0_ein

  length=1.0d0 !unit: a_0=abohr
  if (pot=='EE') length=2.0d0 !unit: a_0=2.0 abohr


!in-file 
  open(90, file=in_file, status='replace')
  write(90,005) pot,1.0/(rd/length),zeff,pol/length**3,r0/length
005 format (A2,/'10000',/'30.0',/F8.3,/F8.3,/F8.3,/F8.3,/'0.001',/'5.0e-01',/'5.0e-02')
  close(90, status="keep")


!part of name of outputfile
  mag=int(dlog10(rd))
  if (mag<6) lenhstr=5
  if (mag>=6) lenhstr=mag
  
  call mkstr(rd,lenhstr,hstr)
  if (pot=='EE'.or.pot=='DB') call mkstr(z,4,zstr)
  if (pot=='PP') then
     call mkstr(pol,6,polstr)
     call mkstr(r0,4,r0str)
  end if

!tinker file
  abbruch=0
  vstr=pot
  lenfstr1=min(len(q)-lenhstr,len(qdb))
  if (vstr=='DB') fstr1=qdb(1:lenfstr1)//zstr//'-'//hstr
  if (vstr=='EE') fstr1=qee(1:lenfstr1)//zstr//'-'//hstr
  if (vstr=='PP') fstr1=qpp(1:lenfstr1)//polstr//'-'//r0str//'-'//hstr
  if (vstr=='PP' .and. rd>1.0d2) &
     fstr1=qpp(1:lenfstr1)//polstr//'-'//r0str//'-'//'100.0'
  q=fstr1

!open of outputfile
  fstr=zielvz//pot//'/'//fstr1 

  error=0
  open(80,file=fstr,iostat=error,status='new',access='sequential')
  if (error/=0) then 
     close(80,status='keep')
     abbruch=1
  else
     write(*,020) fstr1
     call mainei
  end if

020 format &
         &('    ',A,':',/&
         & '     File for transport cross section not existent. The',/&
         & '     calculation might take a while.')



!****************** if files not existent: calculation trq  *****************

  if (abbruch==0) then 

     !parameter for T-matrix calculation
     anzk=0
     test=0

     allocate (matrix(31000,2))
     matrix=0.0d0
     open(10,file=k_l_file, status='old', access='sequential')
     do while (test>=0.0d0)
        read(10,*) (matrix(anzk+1,column),column=1,2)
        test=matrix(anzk+1,1)
        anzk=anzk+1
     end do
     close(10, status='keep')
     
     anzk=anzk-2
     
     allocate(k_tm(0:anzk))
     allocate(k_l_max(0:anzk))
     allocate(qt_tm(1:anzk))
     k_tm(0:anzk)=matrix(1:anzk+1,1)
     k_l_max(0:anzk)=nint(matrix(1:anzk+1,2))
     deallocate(matrix)
     
     kmin=k_tm(1)
     kgrenz=k_tm(anzk)
     l_max=maxval(k_l_max(1:anzk))
     
     hilfanz=1
     
     !write parametr
     text1='# Transportquerschnitte in Abor^2 zum'
     text2='# Potential '
     text3='# rd/a_bohr='
     text4='# Z_eff='
     text5='# pol/a_bohr^3='
     text6='# r_0='
     write(80,022)text2,vstr
     if (vstr=='DB'.or.vstr=='EE') then
        write(80,023)text3,RD,text4,Z
     end if
     if (vstr=='PP') then
        write(80,024)text3,RD,text5,pol,text6,r0
     end if
     
     write(80,*)'#',anzk
     
021  format(A40)
022  format(A12,X,A2)
023  format(A14,F8.3,x,A8,F8.3)
024  format(A14,F8.3,x,A15,F8.3,x,A6,F8.3)
     
     !reading of scattering phase shifts
     allocate(phasen(1:anzk,0:l_max))
     phasen=0.0d0

     !phases in steps of 1
     allocate(matrix(0:anzk,-1:50))
     matrix=0.0d0
     open(10,file=phasenfile(1), status='old', access='sequential')
     do i=0,anzk
        read(10,*)(matrix(i,column),column=-1,min(k_l_max(i),50))
     end do
     close(10, status='keep')
     phasen(1:anzk,0:min(l_max,50))=matrix(1:anzk,0:min(l_max,50))
     deallocate (matrix)
     
     if (50<l_max) then
        do while(k_l_max(hilfanz) < 50)
           hilfanz=hilfanz+1
        end do
        allocate(matrix(hilfanz:anzk,50:100))
        matrix=0.0d0
        open(10,file=phasenfile(2), status='old', access='sequential')
        do i=hilfanz,anzk
           if (k_l_max(i)<50) then 
              matrix(i,50)=k_tm(i)
           else
              read(10,*)(matrix(i,column),column=50,min(k_l_max(i),100))
           end if
        end do
        close(10, status='keep')
        phasen(hilfanz:anzk,51:min(l_max,100))&
             =matrix(hilfanz:anzk,51:min(l_max,100))
        deallocate(matrix)
     end if

     if (100<l_max) then
        do while(k_l_max(hilfanz) < 100)
           hilfanz=hilfanz+1
        end do
        allocate(matrix(hilfanz:anzk,100:150))
        matrix=0.0d0
        open(10,file=phasenfile(3), status='old', access='sequential')
        do i=hilfanz,anzk
           if (k_l_max(i)<100) then 
              matrix(i,100)=k_tm(i)
           else
              read(10,*)(matrix(i,column),column=100,min(k_l_max(i),150))
           end if
        end do
        close(10, status='keep')
        phasen(hilfanz:anzk,101:min(l_max,150))&
             =matrix(hilfanz:anzk,101:min(l_max,150))
        deallocate(matrix)
     end if
     
     
     if (150<l_max) then
        do while(k_l_max(hilfanz) < 150)
           hilfanz=hilfanz+1
        end do
        allocate(matrix(hilfanz:anzk,150:200))
        matrix=0.0d0
        open(10,file=phasenfile(4), status='old', access='sequential')
        do i=hilfanz,anzk
           if (k_l_max(i)<150) then 
              matrix(i,150)=k_tm(i)
           else
              read(10,*)(matrix(i,column),column=150,min(k_l_max(i),200))
           end if
        end do
        close(10, status='keep')
        phasen(hilfanz:anzk,151:min(l_max,200))&
             =matrix(hilfanz:anzk,151:min(l_max,200))
        deallocate(matrix)
     end if
     
     
     !phases in steps of 10
     if (200<l_max) then
        do while(k_l_max(hilfanz) < 200)
           hilfanz=hilfanz+1
        end do
        allocate(phasen10(1:anzk,20:min(l_max/10,100)))
        phasen10=0.0d0
        allocate(matrix(hilfanz:anzk,20:50))
        matrix=0.0d0
        open(10,file=phasenfile(5), status='old', access='sequential')
        do i=hilfanz,anzk
           if (k_l_max(i)<200) then 
              matrix(i,20)=k_tm(i)
           else
              read(10,*)(matrix(i,column),column=20,min(k_l_max(i)/10,50))
           end if
        end do
        close(10, status='keep')
        phasen10(hilfanz:anzk,21:min(l_max/10,50))&
             =matrix(hilfanz:anzk,21:min(l_max/10,50))
        deallocate(matrix)
     end if
     
     if (500<l_max) then
        do while(k_l_max(hilfanz) < 500)
           hilfanz=hilfanz+1
        end do
        allocate(matrix(hilfanz:anzk,50:100))
        matrix=0.0d0
        open(10,file=phasenfile(6), status='old', access='sequential')
        do i=hilfanz,anzk
           if (k_l_max(i)<500) then 
              matrix(i,50)=k_tm(i)
           else
              read(10,*)(matrix(i,column),column=50,min(k_l_max(i)/10,100))
           end if
        end do
        close(10, status='keep')
        phasen10(hilfanz:anzk,51:min(l_max/10,100))=matrix(hilfanz:anzk,51:min(l_max/10,100))
        deallocate(matrix)
     end if
     
     
     if (allocated(phasen10)) then
        phasen10(1:anzk,20)=phasen(1:anzk,200)
        do i=1,anzk
           do j=201,min(l_max,1000)
              hilf=int(j/10)
              if (hilf*10==j) then
                 phasen(i,j)=phasen10(i,hilf)
              else
                 phasen(i,j)=phasen10(i,hilf)&
                      +(j-hilf*10)/1.0d1*(phasen10(i,hilf+1)-phasen10(i,hilf))
              end if
           end do
        end do
     deallocate (phasen10)
  end if
  
  
  !phases in steps of 100
  if (1000<l_max) then
     do while(k_l_max(hilfanz)<1000)
        hilfanz=hilfanz+1
     end do
     allocate(phasen100(1:anzk,10:min(l_max/100,100)))
     phasen100=0.0d0
     allocate(matrix(hilfanz:anzk,10:50))
     matrix=0.0d0
     open(10,file=phasenfile(7), status='old', access='sequential')
     do i=hilfanz,anzk
        if (k_l_max(i)<1000) then 
           matrix(i,10)=k_tm(i)
        else
           read(10,*)(matrix(i,column),column=10,min(k_l_max(i)/100,50))
        end if
     end do
     close(10, status='keep')
     phasen100(hilfanz:anzk,11:min(l_max/100,50))&
          =matrix(hilfanz:anzk,11:min(l_max/100,50))
     deallocate(matrix)
  end if
  
  if (5000<l_max) then
     do while(k_l_max(hilfanz)< 5000)
        hilfanz=hilfanz+1
     end do
     allocate(matrix(hilfanz:anzk,50:100))
     matrix=0.0d0
     open(10,file=phasenfile(8), status='old', access='sequential')
     do i=hilfanz,anzk
        if (k_l_max(i)<5000) then 
           matrix(i,50)=k_tm(i)
        else
           read(10,*)(matrix(i,column),column=50,min(k_l_max(i)/100,100))
        end if
     end do
     close(10, status='keep')
     phasen100(hilfanz:anzk,51:min(l_max/100,100))&
          =matrix(hilfanz:anzk,51:min(l_max/100,100))
     deallocate(matrix)
  end if

  if (allocated(phasen100)) then
     phasen100(1:anzk,10)=phasen(1:anzk,1000)
     do i=1,anzk
        do j=1001,min(l_max,10000)
           hilf=int(j/100)
           if (hilf*100==j) then
              phasen(i,j)=phasen100(i,hilf)
           else
              phasen(i,j)=phasen100(i,hilf)&
                   +(j-hilf*100)/1.0d2*(phasen100(i,hilf+1)-phasen100(i,hilf))
           end if
        end do
     end do
     deallocate (phasen100)
  end if
  
  !ready with reading / deleting phase files
  do i=1,8
     open(10,file=phasenfile(i), status='old', access='sequential')
     close(10, status='delete')
  end do
  
  
  !calculation of trq
  allocate(delta(1:anzk,0:l_max))
  delta=0.0d0

  !e-i and e-a
  if (pot=='DB' .or. pot=='PP') then
     delta=phasen
     delta(1:anzk,0:l_max-1)=delta(1:anzk,0:l_max-1)-phasen(1:anzk,1:l_max)
     delta=dsin(delta)
     delta=delta*delta
     
     qt_tm=0.0d0
     do i=1,anzk
        do j=0,k_l_max(i)
           j_r=real(j)
           vorf=j_r+1.0
           qt_tm(i)=qt_tm(i)+vorf*delta(i,j)
        end do
     end do
     deallocate(delta)
     qt_tm(1:anzk)=4.0d0*pi/k_tm(1:anzk)**2*qt_tm
  end if

  !e-e
  if (pot=='EE') then
     delta=phasen
     delta(1:anzk,0:l_max-2)=delta(1:anzk,0:l_max-2)-phasen(1:anzk,2:l_max)
     delta=dsin(delta)
     delta=delta*delta
     
     qt_tm=0.0d0
     do i=1,anzk
        do j=0,k_l_max(i)
           j_r=real(j)
           vorf=(j_r+1.0)*(j_r+2.0)/(2.0*j_r+3.0)*(1.0-(-1.0)**j/2.0)
           qt_tm(i)=qt_tm(i)+vorf*delta(i,j)
        end do
     end do
     deallocate(delta)
     qt_tm(1:anzk)=4.0d0*pi/k_tm(1:anzk)**2*qt_tm 
  end if
  
  k_tm=k_tm/length
  qt_tm=qt_tm*length**2
  do i=1,anzk
     write(80,060), k_tm(i), qt_tm(i)
  end do
  close(80, status='keep')
  
060 format(F12.6,2x,E15.7)
end if


!******************************* subroutines **********************************
contains

subroutine mkstr(number, length, str)
!***************************************************************************** 
!converts numbers to strings str of lenght characters
!***************************************************************************** 

!in- and output
  double precision, intent(in):: number
  integer, intent(in)::length
  character(len=length), intent(out)::str

!helping variables
  integer, parameter:: i0=iachar('0'), idot=iachar('.')
  integer i
  integer, dimension(1:length):: ziffer
  integer groesse
  double precision hilf, lg

!magnitude of number
  lg=dlog10(number)
  groesse=int(lg)
  if (lg<0 .and. lg==groesse) groesse=groesse+1

!error
  if (abs(groesse)>length .and. groesse>=0) then
     write(*, 010) number, length
     stop
  end if
  if (abs(groesse)+2>=length .and. groesse<0) then
     write(*, 020) number, length
     stop
  end if

010  format&
       &(''/&
       &'error in mkstr:'/&
       &'   Number',F12.3,' is larger than possible for length',I2,'. '/&
       &'   The program is terminated')
020  format&
       &(''/&
       &'error in mkstr:'/&
       &'   Number',F12.3,' is smaller than possible for length',I2,'. '/&
       &'   The program is terminated')


  !initialization
    ziffer=0
    if (groesse >= 0) hilf=number/10**groesse
    if (groesse < 0) hilf=number*10**((-1)*groesse+1)

  !digit to string
    do i=1,length
       ziffer(i)=int(hilf)
       hilf=(hilf-int(hilf))*10.0
       str(i:i)=achar(i0+ziffer(i))
    end do

  !determine string
    if (groesse>=0) then
       str(groesse+3:length)=str(groesse+2:length-1)
       str(groesse+2:groesse+2)=achar(idot)
    else
       str(2-(groesse-1):length)=str(1:length-(2-(groesse-1)))
       do i=1,2-groesse
          str(i:i)=achar(i0)
       end do
       str(1:1)=achar(i0)
       str(2:2)=achar(idot)
    end if
  end subroutine mkstr

!******************************************************************************
end subroutine mktrq

